@@ -1,13 +1,13 @@
 /*
    Definitions of key bindings.
 
-   Copyright (C) 2009, 2011
+   Copyright (C) 2005, 2009, 2010, 2011, 2011, 2012
    The Free Software Foundation, Inc.
 
    Written by:
    Vitja Makarov, 2005
-   Ilia Maslakov <il.smind@gmail.com>, 2009
-   Andrew Borodin <aborodin@vmail.ru>, 2009, 2010
+   Ilia Maslakov <il.smind@gmail.com>, 2009, 2012
+   Andrew Borodin <aborodin@vmail.ru>, 2009, 2010, 2011, 2012
 
    This file is part of the Midnight Commander.
 
@@ -46,6 +46,7 @@
 
 static name_keymap_t command_names[] = {
     /* common */
+    {"InsertChar", CK_InsertChar},
     {"Enter", CK_Enter},
     {"Up", CK_Up},
     {"Down", CK_Down},
@@ -75,7 +76,6 @@ static name_keymap_t command_names[] = {
     {"ChangeOwnAdvanced", CK_ChangeOwnAdvanced},
     {"Remove", CK_Remove},
     {"BackSpace", CK_BackSpace},
-    {"Undo", CK_Undo},
     {"Redo", CK_Redo},
     {"Clear", CK_Clear},
     {"Menu", CK_Menu},
@@ -111,6 +111,8 @@ static name_keymap_t command_names[] = {
     {"ScreenPrev", CK_ScreenPrev},
     {"FileNext", CK_FileNext},
     {"FilePrev", CK_FilePrev},
+    {"DeleteToHome", CK_DeleteToHome},
+    {"DeleteToEnd", CK_DeleteToEnd},
     {"DeleteToWordBegin", CK_DeleteToWordBegin},
     {"DeleteToWordEnd", CK_DeleteToWordEnd},
     {"Cut", CK_Cut},
@@ -163,11 +165,14 @@ static name_keymap_t command_names[] = {
 #ifdef ENABLE_VFS_FTP
     {"ConnectFtp", CK_ConnectFtp},
 #endif
+#ifdef ENABLE_VFS_SFTP
+    {"ConnectSftp", CK_ConnectSftp},
+#endif
 #ifdef ENABLE_VFS_SMB
     {"ConnectSmb", CK_ConnectSmb},
 #endif
     {"PanelInfo", CK_PanelInfo},
-#ifdef WITH_BACKGROUND
+#ifdef ENABLE_BACKGROUND
     {"Jobs", CK_Jobs},
 #endif
     {"OptionsLayout", CK_OptionsLayout},
@@ -194,6 +199,9 @@ static name_keymap_t command_names[] = {
     {"HotListAdd", CK_HotListAdd},
     {"ShowHidden", CK_ShowHidden},
     {"SplitVertHoriz", CK_SplitVertHoriz},
+    {"SplitEqual", CK_SplitEqual},
+    {"SplitMore", CK_SplitMore},
+    {"SplitLess", CK_SplitLess},
     {"PutCurrentPath", CK_PutCurrentPath},
     {"PutOtherPath", CK_PutOtherPath},
     {"PutCurrentTagged", CK_PutCurrentTagged},
@@ -202,6 +210,8 @@ static name_keymap_t command_names[] = {
     {"Unselect", CK_Unselect},
 
     /* panel */
+    {"ScrollLeft", CK_ScrollLeft},
+    {"ScrollRight", CK_ScrollRight},
     {"PanelOtherCd", CK_PanelOtherCd},
     {"PanelOtherCdLink", CK_PanelOtherCdLink},
     {"CopySingle", CK_CopySingle},
@@ -243,7 +253,9 @@ static name_keymap_t command_names[] = {
 #endif
 
 #ifdef USE_INTERNAL_EDIT
+    {"Close", CK_Close},
     {"Tab", CK_Tab},
+    {"Undo", CK_Undo},
     {"ScrollUp", CK_ScrollUp},
     {"ScrollDown", CK_ScrollDown},
     {"Return", CK_Return},
@@ -260,8 +272,6 @@ static name_keymap_t command_names[] = {
     {"InsertOverwrite", CK_InsertOverwrite},
     {"Date", CK_Date},
     {"DeleteLine", CK_DeleteLine},
-    {"DeleteToHome", CK_DeleteToHome},
-    {"DeleteToEnd", CK_DeleteToEnd},
     {"EditMail", CK_Mail},
     {"ParagraphFormat", CK_ParagraphFormat},
     {"MatchBracket", CK_MatchBracket},
@@ -271,6 +281,11 @@ static name_keymap_t command_names[] = {
     {"MacroStartStopRecord", CK_MacroStartStopRecord},
     {"MacroDelete", CK_MacroDelete},
     {"RepeatStartStopRecord", CK_RepeatStartStopRecord},
+#ifdef HAVE_ASPELL
+    {"SpellCheck", CK_SpellCheck},
+    {"SpellCheckCurrentWord", CK_SpellCheckCurrentWord},
+    {"SpellCheckSelectLang", CK_SpellCheckSelectLang},
+#endif /* HAVE_ASPELL */
     {"BookmarkFlush", CK_BookmarkFlush},
     {"BookmarkNext", CK_BookmarkNext},
     {"BookmarkPrev", CK_BookmarkPrev},
@@ -305,6 +320,12 @@ static name_keymap_t command_names[] = {
     {"About", CK_About},
     /* An action to run external script from macro */
     {"ExecuteScript", CK_PipeBlock (0)},
+    {"WindowMove", CK_WindowMove},
+    {"WindowResize", CK_WindowResize},
+    {"WindowFullscreen", CK_WindowFullscreen},
+    {"WindowList", CK_WindowList},
+    {"WindowNext", CK_WindowNext},
+    {"WindowPrev", CK_WindowPrev},
 #endif /* USE_INTERNAL_EDIT */
 
     /* viewer */
@@ -315,14 +336,15 @@ static name_keymap_t command_names[] = {
     {"NroffMode", CK_NroffMode},
     {"BookmarkGoto", CK_BookmarkGoto},
     {"Ruler", CK_Ruler},
+    {"SearchForward", CK_SearchForward},
+    {"SearchBackward", CK_SearchBackward},
+    {"SearchForwardContinue", CK_SearchForwardContinue},
+    {"SearchBackwardContinue", CK_SearchBackwardContinue},
 
 #ifdef USE_DIFF_VIEW
     /* diff viewer */
     {"ShowSymbols", CK_ShowSymbols},
     {"SplitFull", CK_SplitFull},
-    {"SplitEqual", CK_SplitEqual},
-    {"SplitMore", CK_SplitMore},
-    {"SplitLess", CK_SplitLess},
     {"Tab2", CK_Tab2},
     {"Tab3", CK_Tab3},
     {"Tab4", CK_Tab4},
