@@ -80,12 +80,14 @@ enum
     CK_SearchStop,
     CK_Help,
     CK_Edit,
+    CK_EditNew,
     CK_Shell,
     CK_SelectCodepage,
     CK_History,
     CK_HistoryNext,
     CK_HistoryPrev,
     CK_Complete,
+    CK_Save,
     CK_SaveAs,
     CK_Goto,
     CK_Reread,
@@ -99,6 +101,8 @@ enum
     CK_ScreenPrev,
     CK_FilePrev,
     CK_FileNext,
+    CK_DeleteToHome,
+    CK_DeleteToEnd,
     CK_DeleteToWordBegin,
     CK_DeleteToWordEnd,
     CK_ShowNumbers,
@@ -146,6 +150,7 @@ enum
     CK_Filter,
     CK_ConnectFish,
     CK_ConnectFtp,
+    CK_ConnectSftp,
     CK_ConnectSmb,
     CK_PanelInfo,
     CK_Jobs,
@@ -164,6 +169,9 @@ enum
     CK_Tree,
     CK_Undelete,
     CK_SplitVertHoriz,
+    CK_SplitEqual,
+    CK_SplitMore,
+    CK_SplitLess,
     CK_CompareFiles,
     CK_OptionsPanel,
     CK_LinkSymbolicRelative,
@@ -193,6 +201,8 @@ enum
     CK_SortByExt,
     CK_SortBySize,
     CK_SortByMTime,
+    CK_ScrollLeft,
+    CK_ScrollRight,
 
     /* dialog */
     CK_Ok = 300,
@@ -222,18 +232,15 @@ enum
     CK_ParagraphUp,
     CK_ParagraphDown,
     /* file commands */
-    CK_Save,
     CK_EditFile,
-    CK_EditNew,
     CK_InsertFile,
     CK_EditSyntaxFile,
+    CK_Close,
     /* block commands */
     CK_BlockSave,
     CK_BlockShiftLeft,
     CK_BlockShiftRight,
     CK_DeleteLine,
-    CK_DeleteToHome,
-    CK_DeleteToEnd,
     /* bookmarks */
     CK_BookmarkFlush,
     CK_BookmarkNext,
@@ -273,7 +280,17 @@ enum
     CK_RepeatStartRecord,
     CK_RepeatStopRecord,
     CK_RepeatStartStopRecord,
+    /* window commands */
+    CK_WindowMove,
+    CK_WindowResize,
+    CK_WindowFullscreen,
+    CK_WindowList,
+    CK_WindowNext,
+    CK_WindowPrev,
     /* misc commands */
+    CK_SpellCheck,
+    CK_SpellCheckCurrentWord,
+    CK_SpellCheckSelectLang,
     CK_InsertOverwrite,
     CK_ParagraphFormat,
     CK_MatchBracket,
@@ -296,13 +313,14 @@ enum
     CK_HexEditMode,
     CK_BookmarkGoto,
     CK_Ruler,
+    CK_SearchForward,
+    CK_SearchBackward,
+    CK_SearchForwardContinue,
+    CK_SearchBackwardContinue,
 
     /* diff viewer */
     CK_ShowSymbols = 700,
     CK_SplitFull,
-    CK_SplitEqual,
-    CK_SplitMore,
-    CK_SplitLess,
     CK_Tab2,
     CK_Tab3,
     CK_Tab4,
