@@ -32,8 +32,9 @@
 #include "lib/global.h"
 #include "lib/strutil.h"
 #include "lib/search.h"
-
+#ifdef HAVE_CHARSET
 #include "lib/charsets.h"
+#endif
 
 #include "internal.h"
 
@@ -46,7 +47,7 @@
 /*** file scope variables ************************************************************************/
 
 static const mc_search_type_str_t mc_search__list_types[] = {
-    {N_("&Normal"), MC_SEARCH_T_NORMAL},
+    {N_("No&rmal"), MC_SEARCH_T_NORMAL},
     {N_("Re&gular expression"), MC_SEARCH_T_REGEX},
     {N_("He&xadecimal"), MC_SEARCH_T_HEX},
     {N_("Wil&dcard search"), MC_SEARCH_T_GLOB},
