@@ -5,7 +5,9 @@
 
 #include <sys/types.h>
 #include <string.h>
+#ifdef HAVE_ASSERT_H
 #include <assert.h>             /* assert() */
+#endif
 
 /* Header file for strutil.c, strutilascii.c, strutil8bit.c, strutilutf8.c.
  * There are two sort of functions:
@@ -558,7 +560,9 @@ str_move (char *dest, const char *src)
 {
     size_t n;
 
+#ifdef HAVE_ASSERT_H
     assert (dest <= src);
+#endif
 
     n = strlen (src) + 1;       /* + '\0' */
 
