@@ -260,7 +260,6 @@ str_ascii_fit_to_term (const char *text, int width, align_crt_t just_mode)
                 goto finally;
             memset (actual, ' ', width - length - ident);
             actual += width - length - ident;
-            remain -= width - length - ident;
         }
     }
     else
@@ -340,7 +339,6 @@ str_ascii_term_trim (const char *text, int width)
             {
                 memset (actual, '.', width);
                 actual += width;
-                remain -= width;
             }
             else
             {
