@@ -14,6 +14,7 @@
 #include "lib/global.h"
 #include "lib/hook.h"           /* hook_t */
 #include "lib/keybind.h"        /* global_keymap_t */
+#include "lib/tty/mouse.h"      /* mouse_h */
 
 /*** defined constants ***************************************************************************/
 
@@ -24,7 +25,7 @@
 #define B_HELP          3
 #define B_USER          100
 
-#define dlg_move(h, _y, _x) tty_gotoyx (((Dlg_head *)(h))->y + _y, ((Dlg_head *)(h))->x + _x)
+#define dlg_move(h, _y, _x) tty_gotoyx (((Dlg_head *)(h))->y + (_y), ((Dlg_head *)(h))->x + (_x))
 
 /*** enums ***************************************************************************************/
 
@@ -62,7 +63,7 @@ typedef enum
 /* Dialog state */
 typedef enum
 {
-    DLG_CONSTRUCT = 0,          /* DIalog has been constructed but bot run yet */
+    DLG_CONSTRUCT = 0,          /* Dialog has been constructed but not run yet */
     DLG_ACTIVE = 1,             /* Dialog is visible and active */
     DLG_SUSPENDED = 2,          /* Dialog is suspended */
     DLG_CLOSED = 3              /* Dialog is closed */
@@ -128,6 +129,7 @@ struct Dlg_head
     char *event_group;          /* Name of event group for this dialog */
 
     dlg_cb_fn callback;
+    mouse_h mouse;
     dlg_shortcut_str get_shortcut;      /* Shortcut string */
     dlg_title_str get_title;    /* useless for modal dialogs */
 };
@@ -155,7 +157,7 @@ void draw_box (Dlg_head * h, int y, int x, int ys, int xs, gboolean single);
 
 /* Creates a dialog head  */
 Dlg_head *create_dlg (gboolean modal, int y1, int x1, int lines, int cols,
-                      const int *colors, dlg_cb_fn callback,
+                      const int *colors, dlg_cb_fn callback, mouse_h mouse_handler,
                       const char *help_ctx, const char *title, dlg_flags_t flags);
 
 void dlg_set_default_colors (void);
