@@ -2,7 +2,7 @@
    Widgets for the Midnight Commander
 
    Copyright (C) 1994, 1995, 1996, 1998, 1999, 2000, 2001, 2002, 2003,
-   2004, 2005, 2006, 2007, 2009, 2010, 2011
+   2004, 2005, 2006, 2007, 2009, 2010, 2011, 2012
    The Free Software Foundation, Inc.
 
    Authors:
@@ -11,7 +11,7 @@
    Jakub Jelinek, 1995
    Andrej Borsenkow, 1996
    Norbert Warmuth, 1997
-   Andrew Borodin <aborodin@vmail.ru>, 2009, 2010
+   Andrew Borodin <aborodin@vmail.ru>, 2009, 2010, 2011, 2012
 
    This file is part of the Midnight Commander.
 
@@ -287,7 +287,7 @@ history_save (struct mc_config_t *cfg, const char *name, GList * h)
 /* --------------------------------------------------------------------------------------------- */
 
 char *
-history_show (GList ** history, Widget * widget)
+history_show (GList ** history, Widget * widget, int current)
 {
     GList *z, *hlist = NULL, *hi;
     size_t maxlen, i, count = 0;
@@ -320,7 +320,7 @@ history_show (GList ** history, Widget * widget)
     hist_data.maxlen = maxlen;
 
     query_dlg =
-        create_dlg (TRUE, 0, 0, 4, 4, dialog_colors, history_dlg_callback,
+        create_dlg (TRUE, 0, 0, 4, 4, dialog_colors, history_dlg_callback, NULL,
                     "[History-query]", _("History"), DLG_COMPACT);
     query_dlg->data = &hist_data;
 
@@ -342,7 +342,10 @@ history_show (GList ** history, Widget * widget)
     {
         /* draw list entries from bottom upto top */
         listbox_set_list (query_list, hlist);
-        listbox_select_last (query_list);
+        if (current < 0 || (size_t) current >= count)
+            listbox_select_last (query_list);
+        else
+            listbox_select_entry (query_list, count - 1 - (size_t) current);
     }
     else
     {
@@ -350,6 +353,8 @@ history_show (GList ** history, Widget * widget)
         /* revert history direction */
         hlist = g_list_reverse (hlist);
         listbox_set_list (query_list, hlist);
+        if (current > 0)
+            listbox_select_entry (query_list, current);
     }
 
     if (run_dlg (query_dlg) != B_CANCEL)
