@@ -1323,7 +1323,7 @@ complete_engine (WInput * in, int what_to_do)
             query_height = h;
             query_width = w;
             query_dlg = create_dlg (TRUE, y, x, query_height, query_width,
-                                    dialog_colors, query_callback,
+                                    dialog_colors, query_callback, NULL,
                                     "[Completion]", NULL, DLG_COMPACT);
             query_list = listbox_new (1, 1, h - 2, w - 2, FALSE, NULL);
             add_widget (query_dlg, query_list);
