@@ -6,6 +6,8 @@
 #ifndef MC__WIDGET_LISTBOX_H
 #define MC__WIDGET_LISTBOX_H
 
+#include "lib/keybind.h"        /* global_keymap_t */
+
 /*** typedefs(not structures) and defined constants **********************************************/
 
 /*** enums ***************************************************************************************/
@@ -53,6 +55,8 @@ typedef struct WListbox
 
 /*** global variables defined in .c file *********************************************************/
 
+extern const global_keymap_t *listbox_map;
+
 /*** declarations of public functions ************************************************************/
 
 WListbox *listbox_new (int y, int x, int height, int width, gboolean deletable, lcback_fn callback);
