@@ -2,6 +2,7 @@
 #define MC__ARGS_H
 
 #include "lib/global.h"         /* gboolean */
+#include "lib/vfs/vfs.h"        /* vfs_path_t */
 
 /*** typedefs(not structures) and defined constants **********************************************/
 
@@ -9,13 +10,18 @@
 
 /*** structures declarations (and typedefs of structures)*****************************************/
 
+typedef struct
+{
+    vfs_path_t *file_vpath;
+    int line_number;
+} mcedit_arg_t;
+
 /*** global variables defined in .c file *********************************************************/
 
 extern gboolean mc_args__force_xterm;
 extern gboolean mc_args__nomouse;
 extern gboolean mc_args__force_colors;
 extern gboolean mc_args__nokeymap;
-extern int mc_args__edit_start_line;
 extern char *mc_args__last_wd_file;
 extern char *mc_args__netfs_logfile;
 extern char *mc_args__keymap_file;
@@ -27,5 +33,10 @@ gboolean mc_args_parse (int *argc, char ***argv, const char *translation_domain,
 gboolean mc_args_show_info (void);
 gboolean mc_setup_by_args (int argc, char **argv, GError ** error);
 
+mcedit_arg_t *mcedit_arg_new (const char *file_name, int line_number);
+mcedit_arg_t *mcedit_arg_vpath_new (vfs_path_t * file_vpath, int line_number);
+void mcedit_arg_free (mcedit_arg_t * arg);
+
 /*** inline functions ****************************************************************************/
+
 #endif /* MC__ARGS_H */
