@@ -105,14 +105,6 @@
 
 /*** enums ***************************************************************************************/
 
-/* type for file which is currently being edited */
-typedef enum
-{
-    EDIT_FILE_COMMON = 0,
-    EDIT_FILE_SYNTAX = 1,
-    EDIT_FILE_MENU = 2
-} edit_current_file_t;
-
 /* line breaks */
 typedef enum
 {
@@ -175,14 +167,21 @@ extern int option_edit_bottom_extreme;
 extern const char *option_whole_chars_search;
 extern gboolean search_create_bookmark;
 
+extern char *edit_window_state_char;
+extern char *edit_window_close_char;
+
 /*** declarations of public functions ************************************************************/
 
-int edit_drop_hotkey_menu (WEdit * e, int key);
-void edit_menu_cmd (WEdit * e);
+gboolean edit_add_window (Dlg_head * h, int y, int x, int lines, int cols,
+                          const vfs_path_t * f, int fline);
+WEdit *find_editor (const Dlg_head * h);
+gboolean edit_widget_is_editor (const Widget * w);
+gboolean edit_drop_hotkey_menu (Dlg_head * h, int key);
+void edit_menu_cmd (Dlg_head * h);
 void user_menu (WEdit * edit, const char *menu_file, int selected_entry);
 void edit_init_menu (struct WMenuBar *menubar);
-void menu_save_mode_cmd (void);
-int edit_translate_key (WEdit * edit, long x_key, int *cmd, int *ch);
+void edit_save_mode_cmd (void);
+gboolean edit_translate_key (WEdit * edit, long x_key, int *cmd, int *ch);
 int edit_get_byte (WEdit * edit, long byte_index);
 int edit_get_utf (WEdit * edit, long byte_index, int *char_width);
 long edit_count_lines (WEdit * edit, long current, long upto);
@@ -204,7 +203,7 @@ long edit_eol (WEdit * edit, long current);
 void edit_update_curs_row (WEdit * edit);
 void edit_update_curs_col (WEdit * edit);
 void edit_find_bracket (WEdit * edit);
-int edit_reload_line (WEdit * edit, const vfs_path_t * filename_vpath, long line);
+gboolean edit_reload_line (WEdit * edit, const vfs_path_t * filename_vpath, long line);
 void edit_set_codeset (WEdit * edit);
 
 void edit_block_copy_cmd (WEdit * edit);
@@ -213,6 +212,7 @@ int edit_block_delete_cmd (WEdit * edit);
 void edit_delete_line (WEdit * edit);
 
 int edit_delete (WEdit * edit, const int byte_delete);
+int edit_backspace (WEdit * edit, const int byte_delete);
 void edit_insert (WEdit * edit, int c);
 void edit_cursor_move (WEdit * edit, long increment);
 void edit_push_undo_action (WEdit * edit, long c, ...);
@@ -226,12 +226,12 @@ int edit_save_confirm_cmd (WEdit * edit);
 int edit_save_as_cmd (WEdit * edit);
 WEdit *edit_init (WEdit * edit, int y, int x, int lines, int cols,
                   const vfs_path_t * filename_vpath, long line);
-int edit_clean (WEdit * edit);
+gboolean edit_clean (WEdit * edit);
 gboolean edit_ok_to_exit (WEdit * edit);
-int edit_renew (WEdit * edit);
-int edit_new_cmd (WEdit * edit);
-int edit_reload (WEdit * edit, const vfs_path_t * filename_vpath);
-int edit_load_cmd (WEdit * edit, edit_current_file_t what);
+gboolean edit_load_cmd (Dlg_head * h);
+gboolean edit_load_syntax_file (Dlg_head * h);
+gboolean edit_load_menu_file (Dlg_head * h);
+gboolean edit_close_cmd (WEdit * edit);
 void edit_mark_cmd (WEdit * edit, int unmark);
 void edit_mark_current_word_cmd (WEdit * edit);
 void edit_mark_current_line_cmd (WEdit * edit);
@@ -239,32 +239,48 @@ void edit_set_markers (WEdit * edit, long m1, long m2, int c1, int c2);
 void edit_push_markers (WEdit * edit);
 void edit_replace_cmd (WEdit * edit, int again);
 void edit_search_cmd (WEdit * edit, gboolean again);
-int edit_search_cmd_callback (const void *user_data, gsize char_offset);
+mc_search_cbret_t edit_search_cmd_callback (const void *user_data, gsize char_offset,
+                                            int *current_char);
 void edit_complete_word_cmd (WEdit * edit);
 void edit_get_match_keyword_cmd (WEdit * edit);
+
+#ifdef HAVE_ASPELL
+int edit_suggest_current_word (WEdit * edit);
+void edit_spellcheck_file (WEdit * edit);
+void edit_set_spell_lang (void);
+#endif
+
 int edit_save_block (WEdit * edit, const char *filename, long start, long finish);
 int edit_save_block_cmd (WEdit * edit);
 gboolean edit_insert_file_cmd (WEdit * edit);
 void edit_insert_over (WEdit * edit);
 int edit_insert_column_of_text_from_file (WEdit * edit, int file,
                                           long *start_pos, long *end_pos, int *col1, int *col2);
+
+char *edit_get_word_from_pos (WEdit * edit, long start_pos, long *start, gsize * len, gsize * cut);
 long edit_insert_file (WEdit * edit, const vfs_path_t * filename_vpath);
-int edit_load_back_cmd (WEdit * edit);
-int edit_load_forward_cmd (WEdit * edit);
+gboolean edit_load_back_cmd (WEdit * edit);
+gboolean edit_load_forward_cmd (WEdit * edit);
 void edit_block_process_cmd (WEdit * edit, int macro_number);
-void edit_refresh_cmd (WEdit * edit);
+void edit_refresh_cmd (void);
+void edit_syntax_onoff_cmd (Dlg_head * h);
+void edit_show_tabs_tws_cmd (Dlg_head * h);
+void edit_show_margin_cmd (Dlg_head * h);
+void edit_show_numbers_cmd (Dlg_head * h);
 void edit_date_cmd (WEdit * edit);
 void edit_goto_cmd (WEdit * edit);
 int eval_marks (WEdit * edit, long *start_mark, long *end_mark);
-void edit_status (WEdit * edit);
+void edit_status (WEdit * edit, gboolean active);
 void edit_execute_key_command (WEdit * edit, unsigned long command, int char_for_insertion);
 void edit_update_screen (WEdit * edit);
+void edit_save_size (WEdit * edit);
+gboolean edit_handle_move_resize (WEdit * edit, unsigned long command);
+void edit_toggle_fullscreen (WEdit * edit);
 void edit_move_to_line (WEdit * e, long line);
 void edit_move_display (WEdit * e, long line);
 void edit_word_wrap (WEdit * edit);
 int edit_sort_cmd (WEdit * edit);
 int edit_ext_cmd (WEdit * edit);
-void edit_help_cmd (WEdit * edit);
 
 int edit_store_macro_cmd (WEdit * edit);
 gboolean edit_load_macro_cmd (WEdit * edit);
@@ -301,9 +317,10 @@ void book_mark_serialize (WEdit * edit, int color);
 void book_mark_restore (WEdit * edit, int color);
 
 int line_is_blank (WEdit * edit, long line);
+gboolean is_break_char (char c);
 int edit_indent_width (WEdit * edit, long p);
 void edit_insert_indent (WEdit * edit, int indent);
-void edit_options_dialog (WEdit * edit);
+void edit_options_dialog (Dlg_head * h);
 void edit_syntax_dialog (WEdit * edit);
 void edit_mail_dialog (WEdit * edit);
 void format_paragraph (WEdit * edit, int force);
@@ -313,4 +330,17 @@ void edit_execute_cmd (WEdit * edit, unsigned long command, int char_for_inserti
 
 /*** inline functions ****************************************************************************/
 
+/**
+ * Load a new file into the editor.  If it fails, preserve the old file.
+ * To do it, allocate a new widget, initialize it and, if the new file
+ * was loaded, copy the data to the old widget.
+ *
+ * @returns TRUE on success, FALSE on failure.
+ */
+static inline gboolean
+edit_reload (WEdit * edit, const vfs_path_t * filename_vpath)
+{
+    return edit_reload_line (edit, filename_vpath, 0);
+}
+
 #endif /* MC__EDIT_IMPL_H */
