@@ -1,27 +1,26 @@
 /*
    Editor dialogs for high level editing commands
 
-   Copyright (C) 2009 The Free Software Foundation, Inc.
+   Copyright (C) 2009, 2011
+   The Free Software Foundation, Inc.
 
    Written by:
    Slava Zanko <slavazanko@gmail.com>, 2009.
 
    This file is part of the Midnight Commander.
 
-   The Midnight Commander is free software; you can redistribute it
+   The Midnight Commander is free software: you can redistribute it
    and/or modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
+   published by the Free Software Foundation, either version 3 of the License,
+   or (at your option) any later version.
 
-   The Midnight Commander is distributed in the hope that it will be
-   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   The Midnight Commander is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301, USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -33,15 +32,21 @@
 #include "lib/search.h"
 #include "lib/strutil.h"
 #include "lib/widget.h"
+#ifdef HAVE_CHARSET
 #include "lib/charsets.h"
+#endif
 
 #include "src/main.h"
 #include "src/history.h"
 
-#include "src/editor/edit-widget.h"
+#include "src/editor/editwidget.h"
 #include "src/editor/etags.h"
 #include "src/editor/editcmd_dialogs.h"
 
+#ifdef HAVE_ASPELL
+#include "src/editor/spell.h"
+#endif
+
 /*** global variables ****************************************************************************/
 
 edit_search_options_t edit_search_options = {
@@ -106,7 +111,7 @@ editcmd_dialog_replace_show (WEdit * edit, const char *search_default, const cha
             /*  1 */ QUICK_BUTTON (2, 10, 13, REPLACE_DLG_HEIGHT, N_("&OK"), B_ENTER, NULL),
 #ifdef HAVE_CHARSET
             /*  2 */ QUICK_CHECKBOX (33, REPLACE_DLG_WIDTH, 11, REPLACE_DLG_HEIGHT,
-                                     N_("All charsets"),
+                                     N_("&All charsets"),
                                      &edit_search_options.all_codepages),
 #endif
             /*  3 */ QUICK_CHECKBOX (33, REPLACE_DLG_WIDTH, 10, REPLACE_DLG_HEIGHT,
@@ -118,7 +123,7 @@ editcmd_dialog_replace_show (WEdit * edit, const char *search_default, const cha
             /*  5 */ QUICK_CHECKBOX (33, REPLACE_DLG_WIDTH, 8, REPLACE_DLG_HEIGHT, N_("&Backwards"),
                                      &edit_search_options.backwards),
             /*  6 */ QUICK_CHECKBOX (33, REPLACE_DLG_WIDTH, 7, REPLACE_DLG_HEIGHT,
-                                     N_("Case &sensitive"),
+                                     N_("Cas&e sensitive"),
                                      &edit_search_options.case_sens),
             /*  7 */ QUICK_RADIO (3, REPLACE_DLG_WIDTH, 7, REPLACE_DLG_HEIGHT,
                                   num_of_types, (const char **) list_of_types,
@@ -138,7 +143,7 @@ editcmd_dialog_replace_show (WEdit * edit, const char *search_default, const cha
 
         QuickDialog Quick_input = {
             REPLACE_DLG_WIDTH, REPLACE_DLG_HEIGHT, -1, -1, N_("Replace"),
-            "[Input Line Keys]", quick_widgets, NULL, FALSE
+            "[Input Line Keys]", quick_widgets, NULL, NULL, FALSE
         };
 
         if (quick_dialog (&Quick_input) != B_CANCEL)
@@ -178,7 +183,7 @@ editcmd_dialog_search_show (WEdit * edit)
         QUICK_BUTTON (2, 10, 11, SEARCH_DLG_HEIGHT, N_("&OK"), B_ENTER, NULL),
 #ifdef HAVE_CHARSET
         /* 3 */
-        QUICK_CHECKBOX (33, SEARCH_DLG_WIDTH, 9, SEARCH_DLG_HEIGHT, N_("All charsets"),
+        QUICK_CHECKBOX (33, SEARCH_DLG_WIDTH, 9, SEARCH_DLG_HEIGHT, N_("&All charsets"),
                         &edit_search_options.all_codepages),
 #endif
         /* 4 */
@@ -191,7 +196,7 @@ editcmd_dialog_search_show (WEdit * edit)
         QUICK_CHECKBOX (33, SEARCH_DLG_WIDTH, 6, SEARCH_DLG_HEIGHT, N_("&Backwards"),
                         &edit_search_options.backwards),
         /* 7 */
-        QUICK_CHECKBOX (33, SEARCH_DLG_WIDTH, 5, SEARCH_DLG_HEIGHT, N_("Case &sensitive"),
+        QUICK_CHECKBOX (33, SEARCH_DLG_WIDTH, 5, SEARCH_DLG_HEIGHT, N_("Cas&e sensitive"),
                         &edit_search_options.case_sens),
         /* 8 */
         QUICK_RADIO (3, SEARCH_DLG_WIDTH, 5, SEARCH_DLG_HEIGHT,
@@ -214,7 +219,7 @@ editcmd_dialog_search_show (WEdit * edit)
 
     QuickDialog Quick_input = {
         SEARCH_DLG_WIDTH, SEARCH_DLG_HEIGHT, -1, -1, N_("Search"),
-        "[Input Line Keys]", quick_widgets, NULL, TRUE
+        "[Input Line Keys]", quick_widgets, NULL, NULL, TRUE
     };
 
 #ifdef ENABLE_NLS
@@ -265,7 +270,7 @@ editcmd_dialog_search_show (WEdit * edit)
         /* length of buttons */
         b0_len = str_term_width1 (quick_widgets[0].u.button.text) + 3;
         b1_len = str_term_width1 (quick_widgets[1].u.button.text) + 3;
-        b2_len = str_term_width1 (quick_widgets[2].u.button.text) + 5;      /* default button */
+        b2_len = str_term_width1 (quick_widgets[2].u.button.text) + 5;  /* default button */
         len = b0_len + b1_len + b2_len + button_gap * 2;
 
         /* dialog width */
@@ -341,7 +346,7 @@ editcmd_dialog_raw_key_query (const char *heading, const char *query, int cancel
     int w = str_term_width1 (query) + 7;
 
     struct Dlg_head *raw_dlg =
-        create_dlg (TRUE, 0, 0, 7, w, dialog_colors, editcmd_dialog_raw_key_query_cb,
+        create_dlg (TRUE, 0, 0, 7, w, dialog_colors, editcmd_dialog_raw_key_query_cb, NULL,
                     NULL, heading, DLG_CENTER | DLG_TRYUP | DLG_WANT_TAB);
     add_widget (raw_dlg, input_new (3 - cancel, w - 5, input_get_default_colors (),
                                     2, "", 0, INPUT_COMPLETE_DEFAULT));
@@ -350,13 +355,8 @@ editcmd_dialog_raw_key_query (const char *heading, const char *query, int cancel
         add_widget (raw_dlg, button_new (4, w / 2 - 5, B_CANCEL, NORMAL_BUTTON, _("Cancel"), 0));
     w = run_dlg (raw_dlg);
     destroy_dlg (raw_dlg);
-    if (cancel)
-    {
-        if (w == XCTRL ('g') || w == XCTRL ('c') || w == ESC_CHAR || w == B_CANCEL)
-            return 0;
-    }
 
-    return w;
+    return (cancel && (w == ESC_CHAR || w == B_CANCEL)) ? 0 : w;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -378,8 +378,8 @@ editcmd_dialog_completion_show (WEdit * edit, int max_len, int word_len,
     compl_dlg_h = num_compl + 2;
     compl_dlg_w = max_len + 4;
     start_x = edit->curs_col + edit->start_col - (compl_dlg_w / 2) +
-        EDIT_TEXT_HORIZONTAL_OFFSET + option_line_state_width;
-    start_y = edit->curs_row + EDIT_TEXT_VERTICAL_OFFSET + 1;
+        EDIT_TEXT_HORIZONTAL_OFFSET + (edit->fullscreen ? 0 : 1) + option_line_state_width;
+    start_y = edit->curs_row + EDIT_TEXT_VERTICAL_OFFSET + (edit->fullscreen ? 0 : 1) + 1;
 
     if (start_x < 0)
         start_x = 0;
@@ -398,7 +398,7 @@ editcmd_dialog_completion_show (WEdit * edit, int max_len, int word_len,
     /* create the dialog */
     compl_dlg =
         create_dlg (TRUE, start_y, start_x, compl_dlg_h, compl_dlg_w,
-                    dialog_colors, NULL, "[Completion]", NULL, DLG_COMPACT);
+                    dialog_colors, NULL, NULL, "[Completion]", NULL, DLG_COMPACT);
 
     /* create the listbox */
     compl_list = listbox_new (1, 1, compl_dlg_h - 2, compl_dlg_w - 2, FALSE, NULL);
@@ -467,8 +467,8 @@ editcmd_dialog_select_definition_show (WEdit * edit, char *match_expr, int max_l
     def_dlg_h = num_lines + 2;
     def_dlg_w = max_len + 4;
     start_x = edit->curs_col + edit->start_col - (def_dlg_w / 2) +
-        EDIT_TEXT_HORIZONTAL_OFFSET + option_line_state_width;
-    start_y = edit->curs_row + EDIT_TEXT_VERTICAL_OFFSET + 1;
+        EDIT_TEXT_HORIZONTAL_OFFSET + (edit->fullscreen ? 0 : 1) + option_line_state_width;
+    start_y = edit->curs_row + EDIT_TEXT_VERTICAL_OFFSET + (edit->fullscreen ? 0 : 1) + 1;
 
     if (start_x < 0)
         start_x = 0;
@@ -486,7 +486,7 @@ editcmd_dialog_select_definition_show (WEdit * edit, char *match_expr, int max_l
 
     /* create the dialog */
     def_dlg = create_dlg (TRUE, start_y, start_x, def_dlg_h, def_dlg_w,
-                          dialog_colors, NULL, "[Definitions]", match_expr, DLG_COMPACT);
+                          dialog_colors, NULL, NULL, "[Definitions]", match_expr, DLG_COMPACT);
 
     /* create the listbox */
     def_list = listbox_new (1, 1, def_dlg_h - 2, def_dlg_w - 2, FALSE, NULL);
@@ -532,24 +532,25 @@ editcmd_dialog_select_definition_show (WEdit * edit, char *match_expr, int max_l
         {
             if (edit_stack_iterator + 1 < MAX_HISTORY_MOVETO)
             {
-                g_free (edit_history_moveto[edit_stack_iterator].filename);
-                if (edit->dir)
+                vfs_path_free (edit_history_moveto[edit_stack_iterator].filename_vpath);
+                if (edit->dir_vpath != NULL)
                 {
-                    edit_history_moveto[edit_stack_iterator].filename =
-                        g_strdup_printf ("%s/%s", edit->dir, edit->filename);
+                    edit_history_moveto[edit_stack_iterator].filename_vpath =
+                        vfs_path_append_vpath_new (edit->dir_vpath, edit->filename_vpath, NULL);
                 }
                 else
                 {
-                    edit_history_moveto[edit_stack_iterator].filename = g_strdup (edit->filename);
+                    edit_history_moveto[edit_stack_iterator].filename_vpath =
+                        vfs_path_clone (edit->filename_vpath);
                 }
                 edit_history_moveto[edit_stack_iterator].line = edit->start_line +
                     edit->curs_row + 1;
                 edit_stack_iterator++;
-                g_free (edit_history_moveto[edit_stack_iterator].filename);
-                edit_history_moveto[edit_stack_iterator].filename =
-                    g_strdup ((char *) curr_def->fullpath);
+                vfs_path_free (edit_history_moveto[edit_stack_iterator].filename_vpath);
+                edit_history_moveto[edit_stack_iterator].filename_vpath =
+                    vfs_path_from_str ((char *) curr_def->fullpath);
                 edit_history_moveto[edit_stack_iterator].line = curr_def->line;
-                edit_reload_line (edit, edit_history_moveto[edit_stack_iterator].filename,
+                edit_reload_line (edit, edit_history_moveto[edit_stack_iterator].filename_vpath,
                                   edit_history_moveto[edit_stack_iterator].line);
             }
         }
@@ -633,7 +634,7 @@ editcmd_dialog_replace_prompt_show (WEdit * edit, char *from_text, char *to_text
     {
         QuickDialog Quick_input = {
             dlg_width, dlg_height, 0, 0, N_("Confirm replace"),
-            "[Input Line Keys]", quick_widgets, NULL, FALSE
+            "[Input Line Keys]", quick_widgets, NULL, NULL, FALSE
         };
 
         /* Sometimes menu can hide replaced text. I don't like it */
