@@ -2,11 +2,12 @@
    Editor text drawing.
 
    Copyright (C) 1996, 1997, 1998, 2001, 2002, 2003, 2004, 2005, 2006,
-   2007, 2011
+   2007, 2011, 2012
    The Free Software Foundation, Inc.
 
    Written by:
    Paul Sheer, 1996, 1997
+   Andrew Borodin <aborodin@vmail.ru> 2012
 
    This file is part of the Midnight Commander.
 
@@ -47,14 +48,16 @@
 #include "lib/skin.h"
 #include "lib/strutil.h"        /* utf string functions */
 #include "lib/util.h"           /* is_printable() */
-#include "lib/widget.h"         /* buttonbar_redraw() */
+#include "lib/widget.h"
+#ifdef HAVE_CHARSET
 #include "lib/charsets.h"
+#endif
 
 #include "src/setup.h"          /* edit_tab_spacing */
 #include "src/main.h"           /* macro_index */
 
 #include "edit-impl.h"
-#include "edit-widget.h"
+#include "editwidget.h"
 
 /*** global variables ****************************************************************************/
 
@@ -91,6 +94,14 @@ struct line_s
 /*** file scope variables ************************************************************************/
 
 /*** file scope functions ************************************************************************/
+
+static inline void
+printwstr (const char *s, int len)
+{
+    if (len > 0)
+        tty_printf ("%-*.*s", len, len, s);
+}
+
 /* --------------------------------------------------------------------------------------------- */
 
 static inline void
@@ -177,12 +188,198 @@ status_string (WEdit * edit, char *s, int w)
 }
 
 /* --------------------------------------------------------------------------------------------- */
+/**
+ * Draw the status line at the top of the screen for fullscreen editor window.
+ *
+ * @param edit  editor object
+ * @param color color pair
+ */
 
 static inline void
-printwstr (const char *s, int len)
+edit_status_fullscreen (WEdit * edit, int color)
 {
-    if (len > 0)
-        tty_printf ("%-*.*s", len, len, s);
+    const int w = edit->widget.owner->cols;
+    const size_t status_size = w + 1;
+    char *const status = g_malloc (status_size);
+    int status_len;
+    const char *fname = "";
+    int fname_len;
+    const int gap = 3;          /* between the filename and the status */
+    const int right_gap = 5;    /* at the right end of the screen */
+    const int preferred_fname_len = 16;
+
+    status_string (edit, status, status_size);
+    status_len = (int) str_term_width1 (status);
+
+    if (edit->filename_vpath != NULL)
+        fname = x_basename (vfs_path_get_last_path_str (edit->filename_vpath));
+
+    fname_len = str_term_width1 (fname);
+    if (fname_len < preferred_fname_len)
+        fname_len = preferred_fname_len;
+
+    if (fname_len + gap + status_len + right_gap >= w)
+    {
+        if (preferred_fname_len + gap + status_len + right_gap >= w)
+            fname_len = preferred_fname_len;
+        else
+            fname_len = w - (gap + status_len + right_gap);
+        fname = str_trunc (fname, fname_len);
+    }
+
+    dlg_move (edit->widget.owner, 0, 0);
+    tty_setcolor (color);
+    printwstr (fname, fname_len + gap);
+    printwstr (status, w - (fname_len + gap));
+
+    if (simple_statusbar && w > EDITOR_MINIMUM_TERMINAL_WIDTH)
+    {
+        size_t percent = 100;
+
+        if (edit->total_lines + 1 != 0)
+            percent = (edit->curs_line + 1) * 100 / (edit->total_lines + 1);
+        dlg_move (edit->widget.owner, 0, w - 6 - 6);
+        tty_printf (" %3d%%", percent);
+    }
+
+    g_free (status);
+}
+
+/* --------------------------------------------------------------------------------------------- */
+/**
+ * Draw status line for editor window if window is not in fullscreen mode.
+ *
+ * @param edit editor object
+ */
+
+static inline void
+edit_status_window (WEdit * edit)
+{
+    int y, x;
+    int cols = edit->widget.cols;
+
+    tty_setcolor (STATUSBAR_COLOR);
+
+    if (cols > 5)
+    {
+        const char *fname = N_("NoName");
+        char *full_fname = NULL;
+
+        if (edit->filename_vpath != NULL)
+        {
+            full_fname = vfs_path_to_str (edit->filename_vpath);
+            fname = x_basename (full_fname);
+        }
+#ifdef ENABLE_NLS
+        else
+            fname = _(fname);
+#endif
+
+        edit_move (2, 0);
+        tty_printf ("[%s]", str_term_trim (fname, edit->widget.cols - 8 - 6));
+        g_free (full_fname);
+    }
+
+    tty_getyx (&y, &x);
+    x -= edit->widget.x;
+    x += 4;
+    if (x + 6 <= cols - 2 - 6)
+    {
+        edit_move (x, 0);
+        tty_printf ("[%c%c%c%c]",
+                    edit->mark1 != edit->mark2 ? (edit->column_highlight ? 'C' : 'B') : '-',
+                    edit->modified ? 'M' : '-',
+                    macro_index < 0 ? '-' : 'R', edit->overwrite == 0 ? '-' : 'O');
+    }
+
+    if (cols > 30)
+    {
+        edit_move (2, edit->widget.lines - 1);
+        tty_printf ("%3ld %5ld/%ld %6ld/%ld",
+                    edit->curs_col + edit->over_col,
+                    edit->curs_line + 1, edit->total_lines + 1, edit->curs1, edit->last_byte);
+    }
+
+    /*
+     * If we are at the end of file, print <EOF>,
+     * otherwise print the current character as is (if printable),
+     * as decimal and as hex.
+     */
+    if (cols > 46)
+    {
+        edit_move (32, edit->widget.lines - 1);
+        if (edit->curs1 >= edit->last_byte)
+            tty_print_string ("[<EOF>       ]");
+#ifdef HAVE_CHARSET
+        else if (edit->utf8)
+        {
+            unsigned int cur_utf;
+            int cw = 1;
+
+            cur_utf = edit_get_utf (edit, edit->curs1, &cw);
+            if (cw <= 0)
+                cur_utf = edit_get_byte (edit, edit->curs1);
+            tty_printf ("[%05d 0x%04X]", cur_utf, cur_utf);
+        }
+#endif
+        else
+        {
+            unsigned char cur_byte;
+
+            cur_byte = edit_get_byte (edit, edit->curs1);
+            tty_printf ("[%05d 0x%04X]", (unsigned int) cur_byte, (unsigned int) cur_byte);
+        }
+    }
+}
+
+/* --------------------------------------------------------------------------------------------- */
+/**
+ * Draw a frame around edit area.
+ *
+ * @param edit   editor object
+ * @param color  color pair
+ * @param active TRUE if editor object is focused
+ */
+
+static inline void
+edit_draw_frame (const WEdit * edit, int color, gboolean active)
+{
+    const Widget *w = (const Widget *) edit;
+
+    /* draw a frame around edit area */
+    tty_setcolor (color);
+    /* draw double frame for active window if skin supports that */
+    tty_draw_box (w->y, w->x, w->lines, w->cols, !active);
+    /* draw a drag marker */
+    if (edit->drag_state == MCEDIT_DRAG_NORMAL)
+    {
+        tty_setcolor (EDITOR_FRAME_DRAG);
+        widget_move (w, w->lines - 1, w->cols - 1);
+        tty_print_alt_char (ACS_LRCORNER, TRUE);
+    }
+}
+
+/* --------------------------------------------------------------------------------------------- */
+/**
+ * Draw a window control buttons.
+ *
+ * @param edit  editor object
+ * @param color color pair
+ */
+
+static inline void
+edit_draw_window_icons (const WEdit * edit, int color)
+{
+    const Widget *w = (const Widget *) edit;
+    char tmp[17];
+
+    tty_setcolor (color);
+    if (edit->fullscreen)
+        dlg_move (w->owner, 0, w->owner->cols - 6);
+    else
+        widget_move (w, 0, edit->widget.cols - 8);
+    g_snprintf (tmp, sizeof (tmp), "[%s][%s]", edit_window_state_char, edit_window_close_char);
+    tty_print_string (tmp);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -201,6 +398,12 @@ print_to_widget (WEdit * edit, long row, int start_col, int start_col_real,
     int wrap_start;
     int len;
 
+    if (!edit->fullscreen)
+    {
+        x1++;
+        y++;
+    }
+
     tty_setcolor (EDITOR_NORMAL_COLOR);
     if (bookmarked != 0)
         tty_setcolor (bookmarked);
@@ -244,17 +447,15 @@ print_to_widget (WEdit * edit, long row, int start_col, int start_col_real,
     }
 
     edit_move (x1, y);
-    p = line;
     i = 1;
-    while (p->ch)
+    for (p = line; p->ch != 0; p++)
     {
         int style;
         unsigned int textchar;
         int color;
 
-        if (cols_to_skip)
+        if (cols_to_skip != 0)
         {
-            p++;
             cols_to_skip--;
             continue;
         }
@@ -277,41 +478,23 @@ print_to_widget (WEdit * edit, long row, int start_col, int start_col_real,
                 tty_setcolor (EDITOR_MARKED_COLOR);
             }
             else
-            {
-#if 0
-                if (color != EDITOR_NORMAL_COLOR)
-                {
-                    textchar = ' ';
-                    tty_lowlevel_setcolor (color);
-                }
-                else
-#endif
-                    tty_setcolor (EDITOR_WHITESPACE_COLOR);
-            }
+                tty_setcolor (EDITOR_WHITESPACE_COLOR);
         }
+        else if (style & MOD_BOLD)
+            tty_setcolor (EDITOR_BOLD_COLOR);
+        else if (style & MOD_MARKED)
+            tty_setcolor (EDITOR_MARKED_COLOR);
         else
-        {
-            if (style & MOD_BOLD)
-            {
-                tty_setcolor (EDITOR_BOLD_COLOR);
-            }
-            else if (style & MOD_MARKED)
-            {
-                tty_setcolor (EDITOR_MARKED_COLOR);
-            }
-            else
-            {
-                tty_lowlevel_setcolor (color);
-            }
-        }
+            tty_lowlevel_setcolor (color);
+
         if (show_right_margin)
         {
             if (i > option_word_wrap_line_length + edit->start_col)
                 tty_setcolor (EDITOR_RIGHT_MARGIN_COLOR);
             i++;
         }
+
         tty_print_anychar (textchar);
-        p++;
     }
 }
 
@@ -335,7 +518,7 @@ edit_draw_this_line (WEdit * edit, long b, long row, long start_col, long end_co
     int book_mark = 0;
     char line_stat[LINE_STATE_WIDTH + 1] = "\0";
 
-    if (row > edit->widget.lines - 1 - EDIT_TEXT_VERTICAL_OFFSET)
+    if (row > edit->widget.lines - 1 - EDIT_TEXT_VERTICAL_OFFSET - 2 * (edit->fullscreen ? 0 : 1))
         return;
 
     if (book_mark_query_color (edit, edit->start_line + row, BOOK_MARK_COLOR))
@@ -349,10 +532,19 @@ edit_draw_this_line (WEdit * edit, long b, long row, long start_col, long end_co
         abn_style = MOD_ABNORMAL;
 
     end_col -= EDIT_TEXT_HORIZONTAL_OFFSET + option_line_state_width;
+    if (!edit->fullscreen)
+    {
+        const Widget *w = (const Widget *) edit;
+
+        end_col--;
+        if (w->x + w->cols <= w->owner->cols)
+            end_col--;
+    }
 
     edit_get_syntax_color (edit, b - 1, &color);
     q = edit_move_forward3 (edit, b, start_col - edit->start_col, 0);
     start_col_real = (col = (int) edit_move_forward3 (edit, b, 0, q)) + edit->start_col;
+
     if (option_line_state)
     {
         cur_line = edit->start_line + row;
@@ -388,6 +580,9 @@ edit_draw_this_line (WEdit * edit, long b, long row, long start_col, long end_co
             while (col <= end_col - edit->start_col)
             {
                 int cw = 1;
+                int tab_over = 0;
+                gboolean wide_width_char = FALSE;
+                gboolean control_char = FALSE;
 
                 p->ch = 0;
                 p->style = 0;
@@ -437,6 +632,9 @@ edit_draw_this_line (WEdit * edit, long b, long row, long start_col, long end_co
                     break;
                 case '\t':
                     i = TAB_SIZE - ((int) col % TAB_SIZE);
+                    tab_over = (end_col - edit->start_col) - (col + i - 1);
+                    if (tab_over < 0)
+                        i += tab_over;
                     col += i;
                     if (tty_use_colors () &&
                         ((visible_tabs || (visible_tws && q >= tws)) && enable_show_tabs_tws))
@@ -522,12 +720,20 @@ edit_draw_this_line (WEdit * edit, long b, long row, long start_col, long end_co
                         {
                             c = convert_from_8bit_to_utf_c ((unsigned char) c, edit->converter);
                         }
+                        else
+                        {
+                            if (g_unichar_iswide (c))
+                            {
+                                wide_width_char = TRUE;
+                                col++;
+                            }
+                        }
                     }
                     else if (edit->utf8)
                         c = convert_from_utf_to_current_c (c, edit->converter);
                     else
-#endif
                         c = convert_to_display_c (c);
+#endif
 
                     /* Caret notation for control characters */
                     if (c < 32)
@@ -539,6 +745,7 @@ edit_draw_this_line (WEdit * edit, long b, long row, long start_col, long end_co
                         p->style = abn_style;
                         p++;
                         col += 2;
+                        control_char = TRUE;
                         break;
                     }
                     if (c == 127)
@@ -550,6 +757,7 @@ edit_draw_this_line (WEdit * edit, long b, long row, long start_col, long end_co
                         p->style = abn_style;
                         p++;
                         col += 2;
+                        control_char = TRUE;
                         break;
                     }
                     if (!edit->utf8)
@@ -590,6 +798,20 @@ edit_draw_this_line (WEdit * edit, long b, long row, long start_col, long end_co
                 {
                     q += cw - 1;
                 }
+
+                if (col > (end_col - edit->start_col + 1))
+                {
+                    if (wide_width_char)
+                    {
+                        p--;
+                        break;
+                    }
+                    if (control_char)
+                    {
+                        p -= 2;
+                        break;
+                    }
+                }
             }
         }
     }
@@ -598,7 +820,7 @@ edit_draw_this_line (WEdit * edit, long b, long row, long start_col, long end_co
         start_col_real = start_col = 0;
     }
 
-    p->ch = '\0';
+    p->ch = 0;
 
     print_to_widget (edit, row, start_col, start_col_real, end_col, line, line_stat, book_mark);
 }
@@ -787,9 +1009,8 @@ edit_render (WEdit * edit, int page, int row_start, int col_start, int row_end,
     if (page)                   /* if it was an expose event, 'page' would be set */
         edit->force |= REDRAW_PAGE | REDRAW_IN_BOUNDS;
 
-    if (edit->force & REDRAW_COMPLETELY)
-        buttonbar_redraw (find_buttonbar (edit->widget.owner));
     render_edit_text (edit, row_start, col_start, row_end, col_end);
+
     /*
      * edit->force != 0 means a key was pending and the redraw
      * was halted, so next time we must redraw everything in case stuff
@@ -803,60 +1024,29 @@ edit_render (WEdit * edit, int page, int row_start, int col_start, int row_end,
 /*** public functions ****************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
 
-/** Draw the status line at the top of the screen. The size of the filename
- * field varies depending on the width of the screen and the length of
- * the filename. */
 void
-edit_status (WEdit * edit)
+edit_status (WEdit * edit, gboolean active)
 {
-    const int w = edit->widget.owner->cols;
-    const size_t status_size = w + 1;
-    char *const status = g_malloc (status_size);
-    int status_len;
-    const char *fname = "";
-    int fname_len;
-    const int gap = 3;          /* between the filename and the status */
-    const int right_gap = 5;    /* at the right end of the screen */
-    const int preferred_fname_len = 16;
+    int color;
 
-    status_string (edit, status, status_size);
-    status_len = (int) str_term_width1 (status);
-
-    if (edit->filename_vpath != NULL)
-        fname = vfs_path_get_last_path_str (edit->filename_vpath);
-
-    fname_len = str_term_width1 (fname);
-    if (fname_len < preferred_fname_len)
-        fname_len = preferred_fname_len;
-
-    if (fname_len + gap + status_len + right_gap >= w)
+    if (edit->fullscreen)
     {
-        if (preferred_fname_len + gap + status_len + right_gap >= w)
-            fname_len = preferred_fname_len;
-        else
-            fname_len = w - (gap + status_len + right_gap);
-        fname = str_trunc (fname, fname_len);
+        color = STATUSBAR_COLOR;
+        edit_status_fullscreen (edit, color);
+    }
+    else
+    {
+        color = edit->drag_state != MCEDIT_DRAG_NORMAL ? EDITOR_FRAME_DRAG : active ?
+            EDITOR_FRAME_ACTIVE : EDITOR_FRAME;
+        edit_draw_frame (edit, color, active);
+        edit_status_window (edit);
     }
 
-    dlg_move (edit->widget.owner, 0, 0);
-    tty_setcolor (STATUSBAR_COLOR);
-    printwstr (fname, fname_len + gap);
-    printwstr (status, w - (fname_len + gap));
-
-    if (simple_statusbar && w > EDITOR_MINIMUM_TERMINAL_WIDTH)
-    {
-        size_t percent = 100;
-
-        if (edit->total_lines + 1 != 0)
-            percent = (edit->curs_line + 1) * 100 / (edit->total_lines + 1);
-        dlg_move (edit->widget.owner, 0, w - 5);
-        tty_printf (" %3d%%", percent);
-    }
-
-    g_free (status);
+    edit_draw_window_icons (edit, color);
 }
 
 /* --------------------------------------------------------------------------------------------- */
+
 /** this scrolls the text so that cursor is on the screen */
 void
 edit_scroll_screen_over_cursor (WEdit * edit)
@@ -868,8 +1058,16 @@ edit_scroll_screen_over_cursor (WEdit * edit)
     if (edit->widget.lines <= 0 || edit->widget.cols <= 0)
         return;
 
-    edit->widget.cols -= EDIT_TEXT_HORIZONTAL_OFFSET + option_line_state_width;
     edit->widget.lines -= EDIT_TEXT_VERTICAL_OFFSET;
+    edit->widget.cols -= EDIT_TEXT_HORIZONTAL_OFFSET + option_line_state_width;
+
+    if (!edit->fullscreen)
+    {
+        edit->widget.x++;
+        edit->widget.cols -= 2;
+        edit->widget.y++;
+        edit->widget.lines -= 2;
+    }
 
     r_extreme = EDIT_RIGHT_EXTREME;
     l_extreme = EDIT_LEFT_EXTREME;
@@ -919,6 +1117,13 @@ edit_scroll_screen_over_cursor (WEdit * edit)
 
     edit->widget.lines += EDIT_TEXT_VERTICAL_OFFSET;
     edit->widget.cols += EDIT_TEXT_HORIZONTAL_OFFSET + option_line_state_width;
+    if (!edit->fullscreen)
+    {
+        edit->widget.x--;
+        edit->widget.cols += 2;
+        edit->widget.y--;
+        edit->widget.lines += 2;
+    }
 }
 
 /* --------------------------------------------------------------------------------------------- */
