@@ -1,11 +1,12 @@
 /*
    Editor menu definitions and initialisation
 
-   Copyright (C) 1996, 1998, 2001, 2002, 2003, 2005, 2007, 2011
+   Copyright (C) 1996, 1998, 2001, 2002, 2003, 2005, 2007, 2011, 2012
    The Free Software Foundation, Inc.
 
    Written by:
    Paul Sheer, 1996, 1997
+   Andrew Borodin <aborodin@vmail.ru> 2012
 
    This file is part of the Midnight Commander.
 
@@ -51,7 +52,7 @@
 #include "src/keybind-defaults.h"
 
 #include "edit-impl.h"
-#include "edit-widget.h"
+#include "editwidget.h"
 
 /*** global variables ****************************************************************************/
 
@@ -71,6 +72,7 @@ create_file_menu (void)
 
     entries = g_list_prepend (entries, menu_entry_create (_("&Open file..."), CK_EditFile));
     entries = g_list_prepend (entries, menu_entry_create (_("&New"), CK_EditNew));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Close"), CK_Close));
     entries = g_list_prepend (entries, menu_separator_create ());
     entries = g_list_prepend (entries, menu_entry_create (_("&Save"), CK_Save));
     entries = g_list_prepend (entries, menu_entry_create (_("Save &as..."), CK_SaveAs));
@@ -173,10 +175,19 @@ create_command_menu (void)
         g_list_prepend (entries,
                         menu_entry_create (_("Record/Repeat &actions"), CK_RepeatStartStopRecord));
     entries = g_list_prepend (entries, menu_separator_create ());
+#ifdef HAVE_ASPELL
+    entries = g_list_prepend (entries, menu_entry_create (_("S&pell check"), CK_SpellCheck));
     entries =
-        g_list_prepend (entries, menu_entry_create (_("'ispell' s&pell check"), CK_PipeBlock (1)));
+        g_list_prepend (entries, menu_entry_create (_("C&heck word"), CK_SpellCheckCurrentWord));
+    entries =
+        g_list_prepend (entries,
+                        menu_entry_create (_("Change spelling &language"),
+                                           CK_SpellCheckSelectLang));
+    entries = g_list_prepend (entries, menu_separator_create ());
+#endif /* HAVE_ASPELL */
     entries = g_list_prepend (entries, menu_entry_create (_("&Mail..."), CK_Mail));
 
+
     return g_list_reverse (entries);
 }
 
@@ -202,6 +213,28 @@ create_format_menu (void)
     return g_list_reverse (entries);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+/**
+ * Create the 'window' popup menu
+ */
+
+static GList *
+create_window_menu (void)
+{
+    GList *entries = NULL;
+
+    entries = g_list_prepend (entries, menu_entry_create (_("&Move"), CK_WindowMove));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Resize"), CK_WindowResize));
+    entries =
+        g_list_prepend (entries, menu_entry_create (_("&Toggle fullscreen"), CK_WindowFullscreen));
+    entries = g_list_prepend (entries, menu_separator_create ());
+    entries = g_list_prepend (entries, menu_entry_create (_("&Next"), CK_WindowNext));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Previous"), CK_WindowPrev));
+    entries = g_list_prepend (entries, menu_entry_create (_("&List..."), CK_WindowList));
+
+    return g_list_reverse (entries);
+}
+
 /* --------------------------------------------------------------------------------------------- */
 
 static GList *
@@ -226,11 +259,11 @@ create_options_menu (void)
 /* --------------------------------------------------------------------------------------------- */
 
 static void
-edit_drop_menu_cmd (WEdit * e, int which)
+edit_drop_menu_cmd (Dlg_head * h, int which)
 {
     WMenuBar *menubar;
 
-    menubar = find_menubar (e->widget.owner);
+    menubar = find_menubar (h);
 
     if (!menubar->is_active)
     {
@@ -239,7 +272,7 @@ edit_drop_menu_cmd (WEdit * e, int which)
         if (which >= 0)
             menubar->selected = which;
 
-        menubar->previous_widget = dlg_get_current_widget_id (e->widget.owner);
+        menubar->previous_widget = dlg_get_current_widget_id (h);
         dlg_select_widget (menubar);
     }
 }
@@ -263,6 +296,8 @@ edit_init_menu (struct WMenuBar *menubar)
                                    "[Internal File Editor]"));
     menubar_add_menu (menubar,
                       create_menu (_("For&mat"), create_format_menu (), "[Internal File Editor]"));
+    menubar_add_menu (menubar,
+                      create_menu (_("&Window"), create_window_menu (), "[Internal File Editor]"));
     menubar_add_menu (menubar,
                       create_menu (_("&Options"), create_options_menu (),
                                    "[Internal File Editor]"));
@@ -271,15 +306,15 @@ edit_init_menu (struct WMenuBar *menubar)
 /* --------------------------------------------------------------------------------------------- */
 
 void
-edit_menu_cmd (WEdit * e)
+edit_menu_cmd (Dlg_head * h)
 {
-    edit_drop_menu_cmd (e, -1);
+    edit_drop_menu_cmd (h, -1);
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
-int
-edit_drop_hotkey_menu (WEdit * e, int key)
+gboolean
+edit_drop_hotkey_menu (Dlg_head * h, int key)
 {
     int m = 0;
     switch (key)
@@ -299,15 +334,18 @@ edit_drop_hotkey_menu (WEdit * e, int key)
     case ALT ('m'):
         m = 4;
         break;
-    case ALT ('o'):
+    case ALT ('w'):
         m = 5;
         break;
+    case ALT ('o'):
+        m = 6;
+        break;
     default:
-        return 0;
+        return FALSE;
     }
 
-    edit_drop_menu_cmd (e, m);
-    return 1;
+    edit_drop_menu_cmd (h, m);
+    return TRUE;
 }
 
 /* --------------------------------------------------------------------------------------------- */
