@@ -50,7 +50,7 @@
 #include "src/setup.h"          /* option_tab_spacing */
 
 #include "edit-impl.h"
-#include "edit-widget.h"
+#include "editwidget.h"
 
 /*** global variables ****************************************************************************/
 
