@@ -1,20 +1,24 @@
-/* Chown command -- for the Midnight Commander
+/*
+   Chown command -- for the Midnight Commander
+
    Copyright (C) 1994, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-   2007 Free Software Foundation, Inc.
+   2007, 2011
+   The Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   This file is part of the Midnight Commander.
 
-   This program is distributed in the hope that it will be useful,
+   The Midnight Commander is free software: you can redistribute it
+   and/or modify it under the terms of the GNU General Public License as
+   published by the Free Software Foundation, either version 3 of the License,
+   or (at your option) any later version.
+
+   The Midnight Commander is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /** \file chown.c
@@ -36,7 +40,7 @@
 
 #include "lib/tty/tty.h"
 #include "lib/skin.h"
-#include "lib/vfs/mc-vfs/vfs.h"
+#include "lib/vfs/vfs.h"
 #include "lib/strutil.h"
 #include "lib/util.h"
 #include "lib/widget.h"
@@ -46,7 +50,6 @@
 /* Needed for the extern declarations of integer parameters */
 #include "chmod.h"
 #include "midnight.h"           /* current_panel */
-#include "layout.h"             /* repaint_screen() */
 
 #include "chown.h"
 
@@ -174,7 +177,7 @@ init_chown (void)
     single_set = (current_panel->marked < 2) ? 3 : 0;
 
     ch_dlg =
-        create_dlg (TRUE, 0, 0, 18, 74, dialog_colors, chown_callback, "[Chown]",
+        create_dlg (TRUE, 0, 0, 18, 74, dialog_colors, chown_callback, NULL, "[Chown]",
                     _("Chown command"), DLG_CENTER | DLG_REVERSE);
 
     for (i = 0; i < BUTTONS - single_set; i++)
@@ -239,10 +242,14 @@ chown_done (void)
 static void
 do_chown (uid_t u, gid_t g)
 {
-    if (mc_chown (current_panel->dir.list[current_file].fname, u, g) == -1)
+    vfs_path_t *vpath;
+
+    vpath = vfs_path_from_str (current_panel->dir.list[current_file].fname);
+    if (mc_chown (vpath, u, g) == -1)
         message (D_ERROR, MSG_ERROR, _("Cannot chown \"%s\"\n%s"),
                  current_panel->dir.list[current_file].fname, unix_error_string (errno));
 
+    vfs_path_free (vpath);
     do_file_mark (current_panel, current_file, 0);
 }
 
@@ -251,15 +258,13 @@ do_chown (uid_t u, gid_t g)
 static void
 apply_chowns (uid_t u, gid_t g)
 {
-    char *fname;
 
     need_update = end_chown = 1;
     do_chown (u, g);
 
     do
     {
-        fname = next_file ();
-
+        next_file ();
         do_chown (u, g);
     }
     while (current_panel->marked);
@@ -281,6 +286,8 @@ chown_cmd (void)
 
     do
     {                           /* do while any files remaining */
+        vfs_path_t *vpath;
+
         ch_dlg = init_chown ();
         new_user = new_group = -1;
 
@@ -289,11 +296,14 @@ chown_cmd (void)
         else
             fname = selection (current_panel)->fname;   /* single file */
 
-        if (mc_stat (fname, &sf_stat) != 0)
+        vpath = vfs_path_from_str (fname);
+        if (mc_stat (vpath, &sf_stat) != 0)
         {                       /* get status of file */
             destroy_dlg (ch_dlg);
+            vfs_path_free (vpath);
             break;
         }
+        vfs_path_free (vpath);
 
         /* select in listboxes */
         listbox_select_entry (l_user, listbox_search_text (l_user, get_owner (sf_stat.st_uid)));
@@ -359,10 +369,14 @@ chown_cmd (void)
                     new_user = user->pw_uid;
                 if (ch_dlg->ret_value == B_ENTER)
                 {
+                    vfs_path_t *fname_vpath;
+
+                    fname_vpath = vfs_path_from_str (fname);
                     need_update = 1;
-                    if (mc_chown (fname, new_user, new_group) == -1)
+                    if (mc_chown (fname_vpath, new_user, new_group) == -1)
                         message (D_ERROR, MSG_ERROR, _("Cannot chown \"%s\"\n%s"),
                                  fname, unix_error_string (errno));
+                    vfs_path_free (fname_vpath);
                 }
                 else
                     apply_chowns (new_user, new_group);
