@@ -5,7 +5,7 @@
 #ifndef MC__FILE_H
 #define MC__FILE_H
 
-#include <inttypes.h>            /* off_t, uintmax_t */
+#include <inttypes.h>           /* off_t, uintmax_t */
 
 #include "lib/global.h"
 #include "lib/widget.h"
@@ -16,14 +16,13 @@
 
 /* Compute directory size */
 /* callback to update status dialog */
-typedef FileProgressStatus (*compute_dir_size_callback) (const void *ui, const char *dirname);
+typedef FileProgressStatus (*compute_dir_size_callback) (const void *ui,
+                                                         const vfs_path_t * dirname_vpath);
 
 /*** enums ***************************************************************************************/
 
 /*** structures declarations (and typedefs of structures)*****************************************/
 
-struct link;
-
 /* status dialog of directory size computing */
 typedef struct
 {
@@ -42,8 +41,9 @@ FileProgressStatus move_dir_dir (FileOpTotalContext * tctx, FileOpContext * ctx,
 FileProgressStatus copy_dir_dir (FileOpTotalContext * tctx, FileOpContext * ctx,
                                  const char *s, const char *d,
                                  gboolean toplevel, gboolean move_over, gboolean do_delete,
-                                 struct link *parent_dirs);
-FileProgressStatus erase_dir (FileOpTotalContext * tctx, FileOpContext * ctx, const char *s);
+                                 GSList * parent_dirs);
+FileProgressStatus erase_dir (FileOpTotalContext * tctx, FileOpContext * ctx,
+                              const vfs_path_t * vpath);
 
 gboolean panel_operate (void *source_panel, FileOperation op, gboolean force_single);
 
@@ -53,14 +53,14 @@ gboolean panel_operate (void *source_panel, FileOperation op, gboolean force_sin
 FileProgressStatus file_error (const char *format, const char *file);
 
 /* return value is FILE_CONT or FILE_ABORT */
-FileProgressStatus compute_dir_size (const char *dirname, const void *ui,
+FileProgressStatus compute_dir_size (const vfs_path_t * dirname_vpath, const void *ui,
                                      compute_dir_size_callback cback,
-                                     size_t * ret_marked, uintmax_t *ret_total,
+                                     size_t * ret_marked, uintmax_t * ret_total,
                                      gboolean compute_symlinks);
 
 ComputeDirSizeUI *compute_dir_size_create_ui (void);
 void compute_dir_size_destroy_ui (ComputeDirSizeUI * ui);
-FileProgressStatus compute_dir_size_update_ui (const void *ui, const char *dirname);
+FileProgressStatus compute_dir_size_update_ui (const void *ui, const vfs_path_t * dirname_vpath);
 
 /*** inline functions ****************************************************************************/
 #endif /* MC__FILE_H */
