@@ -43,6 +43,8 @@
 #include "lib/util.h"
 #include "lib/vfs/vfs.h"
 
+#include "filenot.h"
+
 /*** global variables ****************************************************************************/
 
 /*** file scope macro definitions ****************************************************************/
@@ -54,15 +56,16 @@
 /*** file scope functions ************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
 
-static char *
-get_absolute_name (const char *file)
+static vfs_path_t *
+get_absolute_name (const vfs_path_t * vpath)
 {
-    char dir[MC_MAXPATHLEN];
+    if (vpath == NULL)
+        return NULL;
 
-    if (file[0] == PATH_SEP)
-        return g_strdup (file);
-    mc_get_current_wd (dir, MC_MAXPATHLEN);
-    return concat_dir_and_file (dir, file);
+    if (*(vfs_path_get_by_index (vpath, 0)->path) == PATH_SEP)
+        return vfs_path_clone (vpath);
+
+    return vfs_path_append_vpath_new (vfs_get_raw_current_dir (), vpath);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -72,32 +75,41 @@ my_mkdir_rec (char *s, mode_t mode)
 {
     char *p, *q;
     int result;
+    vfs_path_t *s_vpath;
 
-    if (!mc_mkdir (s, mode))
+    s_vpath = vfs_path_from_str (s);
+    if (mc_mkdir (s_vpath, mode) == 0)
+    {
+        vfs_path_free (s_vpath);
         return 0;
+    }
     else if (errno != ENOENT)
+    {
+        vfs_path_free (s_vpath);
         return -1;
+    }
 
     /* FIXME: should check instead if s is at the root of that filesystem */
     {
-        vfs_path_t *vpath = vfs_path_from_str (s);
-        if (!vfs_file_is_local (vpath))
+        if (!vfs_file_is_local (s_vpath))
         {
-            vfs_path_free (vpath);
+            vfs_path_free (s_vpath);
             return -1;
         }
-        vfs_path_free (vpath);
     }
 
     if (!strcmp (s, PATH_SEP_STR))
     {
         errno = ENOTDIR;
+        vfs_path_free (s_vpath);
         return -1;
     }
 
-    p = concat_dir_and_file (s, "..");
+    p = mc_build_filename (s, "..", NULL);
     {
-        vfs_path_t *vpath = vfs_path_from_str (p);
+        vfs_path_t *vpath;
+
+        vpath = vfs_path_from_str (p);
         q = vfs_path_to_str (vpath);
         vfs_path_free (vpath);
     }
@@ -105,8 +117,9 @@ my_mkdir_rec (char *s, mode_t mode)
 
     result = my_mkdir_rec (q, mode);
     if (result == 0)
-        result = mc_mkdir (s, mode);
+        result = mc_mkdir (s_vpath, mode);
 
+    vfs_path_free (s_vpath);
     g_free (q);
     return result;
 }
@@ -116,32 +129,29 @@ my_mkdir_rec (char *s, mode_t mode)
 /* --------------------------------------------------------------------------------------------- */
 
 int
-my_mkdir (const char *s, mode_t mode)
+my_mkdir (const vfs_path_t * s_vpath, mode_t mode)
 {
     int result;
-    char *my_s;
 
-    result = mc_mkdir (s, mode);
-    if (result)
+    result = mc_mkdir (s_vpath, mode);
+
+    if (result != 0)
     {
-        vfs_path_t *vpath;
         char *p;
-        vpath = vfs_path_from_str (s);
-        p = vfs_path_to_str (vpath);
 
+        p = vfs_path_to_str (s_vpath);
         result = my_mkdir_rec (p, mode);
-        vfs_path_free (vpath);
         g_free (p);
     }
     if (result == 0)
     {
-        my_s = get_absolute_name (s);
+        vfs_path_t *my_s;
 
+        my_s = get_absolute_name (s_vpath);
 #ifdef FIXME
         tree_add_entry (tree, my_s);
 #endif
-
-        g_free (my_s);
+        vfs_path_free (my_s);
     }
     return result;
 }
@@ -152,23 +162,25 @@ int
 my_rmdir (const char *s)
 {
     int result;
-    char *my_s;
+    vfs_path_t *vpath;
 #ifdef FIXME
     WTree *tree = 0;
 #endif
 
+    vpath = vfs_path_from_str_flags (s, VPF_NO_CANON);
     /* FIXME: Should receive a Wtree! */
-    result = mc_rmdir (s);
+    result = mc_rmdir (vpath);
     if (result == 0)
     {
-        my_s = get_absolute_name (s);
+        vfs_path_t *my_s;
 
+        my_s = get_absolute_name (vpath);
 #ifdef FIXME
         tree_remove_entry (tree, my_s);
 #endif
-
-        g_free (my_s);
+        vfs_path_free (my_s);
     }
+    vfs_path_free (vpath);
     return result;
 }
 
