@@ -504,7 +504,7 @@ hotlist_button_callback (WButton * button, int action)
 
 #ifdef	ENABLE_VFS
     case B_FREE_ALL_VFS:
-        vfs_expire (1);
+        vfs_expire (TRUE);
         /* fall through */
 
     case B_REFRESH_VFS:
@@ -777,7 +777,7 @@ init_hotlist (int list_type)
 
     hotlist_dlg =
         create_dlg (TRUE, 0, 0, LINES - 2, hotlist_cols, dialog_colors,
-                    hotlist_callback, help_node, title, DLG_CENTER | DLG_REVERSE);
+                    hotlist_callback, NULL, help_node, title, DLG_CENTER | DLG_REVERSE);
 
     for (i = 0; i < BUTTONS; i++)
     {
@@ -788,7 +788,8 @@ init_hotlist (int list_type)
                                             hotlist_but[i].ret_cmd,
                                             hotlist_but[i].flags,
                                             hotlist_but[i].text,
-                                            hotlist_button_callback), hotlist_but[i].pos_flags);
+                                            hotlist_button_callback), hotlist_but[i].pos_flags,
+                                NULL);
     }
 
     /* We add the labels.
@@ -796,7 +797,7 @@ init_hotlist (int list_type)
      *    pname_group will hold name of current group
      */
     pname = label_new (UY - 11 + LINES, UX + 2, "");
-    add_widget_autopos (hotlist_dlg, pname, WPOS_KEEP_BOTTOM | WPOS_KEEP_LEFT);
+    add_widget_autopos (hotlist_dlg, pname, WPOS_KEEP_BOTTOM | WPOS_KEEP_LEFT, NULL);
     if (!hotlist_state.moving)
     {
         char label_text[BUF_TINY];
@@ -804,7 +805,7 @@ init_hotlist (int list_type)
         g_snprintf (label_text, sizeof (label_text), " %s ", _("Directory path"));
         add_widget_autopos (hotlist_dlg,
                             label_new (UY - 12 + LINES, UX + 2,
-                                       label_text), WPOS_KEEP_BOTTOM | WPOS_KEEP_LEFT);
+                                       label_text), WPOS_KEEP_BOTTOM | WPOS_KEEP_LEFT, NULL);
 
         /* This one holds the displayed pathname */
         pname_group = label_new (UY, UX + 2, _("Directory label"));
@@ -824,7 +825,7 @@ init_hotlist (int list_type)
 #endif /* !ENABLE_VFS */
         fill_listbox ();
 
-    add_widget_autopos (hotlist_dlg, l_hotlist, WPOS_KEEP_ALL);
+    add_widget_autopos (hotlist_dlg, l_hotlist, WPOS_KEEP_ALL, NULL);
     /* add listbox to the dialogs */
 }
 
@@ -841,7 +842,7 @@ init_movelist (int list_type, struct hotlist *item)
 
     movelist_dlg =
         create_dlg (TRUE, 0, 0, LINES - 6, movelist_cols, dialog_colors,
-                    hotlist_callback, "[Hotlist]", hdr, DLG_CENTER | DLG_REVERSE);
+                    hotlist_callback, NULL, "[Hotlist]", hdr, DLG_CENTER | DLG_REVERSE);
     g_free (hdr);
 
     for (i = 0; i < BUTTONS; i++)
@@ -1061,7 +1062,7 @@ add_new_entry_input (const char *header, const char *text1, const char *text2,
     {
         QuickDialog Quick_input = {
             len, lines1 + lines2 + 7, -1, -1, header,
-            help, quick_widgets, NULL, FALSE
+            help, quick_widgets, NULL, NULL, FALSE
         };
 
         for (i = 0; i < 7; i++)
@@ -1092,7 +1093,7 @@ add_new_entry_cmd (void)
     int ret;
 
     /* Take current directory as default value for input fields */
-    to_free = title = url = strip_password (g_strdup (current_panel->cwd), 1);
+    to_free = title = url = vfs_path_to_str_flags (current_panel->cwd_vpath, 0, VPF_STRIP_PASSWORD);
 
     ret = add_new_entry_input (_("New hotlist entry"), _("Directory label:"),
                                _("Directory path:"), "[Hotlist]", &title, &url);
@@ -1153,7 +1154,7 @@ add_new_group_input (const char *header, const char *label, char **result)
     {
         QuickDialog Quick_input = {
             len, lines + 6, -1, -1, header,
-            "[Hotlist]", quick_widgets, NULL, FALSE
+            "[Hotlist]", quick_widgets, NULL, NULL, FALSE
         };
 
         int relative_y[] = { 1, 1, 1, 0, 2 };   /* the relative_x component from the
@@ -1227,35 +1228,33 @@ remove_from_hotlist (struct hotlist *entry)
 
     if (confirm_directory_hotlist_delete)
     {
-        char *title;
+        char text[BUF_MEDIUM];
         int result;
 
-        title = g_strconcat (_("Remove:"), " ", str_trunc (entry->label, 30), (char *) NULL);
-
         if (safe_delete)
             query_set_sel (1);
-        result = query_dialog (title,
-                               _("Are you sure you want to remove this entry?"),
-                               D_ERROR, 2, _("&Yes"), _("&No"));
-
-        g_free (title);
 
+        g_snprintf (text, sizeof (text), _("Are you sure you want to remove entry \"%s\"?"),
+                    str_trunc (entry->label, 30));
+        result = query_dialog (Q_ ("DialogTitle|Delete"), text, D_ERROR | D_CENTER, 2,
+                               _("&Yes"), _("&No"));
         if (result != 0)
             return;
     }
 
     if (entry->type == HL_TYPE_GROUP)
     {
-        if (entry->head)
+        struct hotlist *head = entry->head;
+
+        if (head != NULL && (head->type != HL_TYPE_DOTDOT || head->next != NULL))
         {
-            char *header;
+            char text[BUF_MEDIUM];
             int result;
 
-            header = g_strconcat (_("Remove:"), " ", str_trunc (entry->label, 30), (char *) NULL);
-            result = query_dialog (header, _("Group not empty.\nRemove it?"),
-                                   D_ERROR, 2, _("&Yes"), _("&No"));
-            g_free (header);
-
+            g_snprintf (text, sizeof (text), _("Group \"%s\" is not empty.\nRemove it?"),
+                        str_trunc (entry->label, 30));
+            result = query_dialog (Q_ ("DialogTitle|Delete"), text, D_ERROR | D_CENTER, 2,
+                                   _("&Yes"), _("&No"));
             if (result != 0)
                 return;
         }
@@ -1434,14 +1433,11 @@ hot_load_group (struct hotlist *grp)
             break;
         case TKN_ENTRY:
             {
-                vfs_path_t *vpath;
                 CHECK_TOKEN (TKN_STRING);
                 label = g_strdup (tkn_buf->str);
                 CHECK_TOKEN (TKN_URL);
                 CHECK_TOKEN (TKN_STRING);
-                vpath = vfs_path_from_str_flags (tkn_buf->str, VPF_USE_DEPRECATED_PARSER);
-                url = vfs_path_to_str (vpath);
-                vfs_path_free (vpath);
+                url = g_strdup (tkn_buf->str);
                 add2hotlist (label, url, HL_TYPE_ENTRY, LISTBOX_APPEND_AT_END);
                 SKIP_TO_EOL;
             }
@@ -1491,14 +1487,11 @@ hot_load_file (struct hotlist *grp)
             break;
         case TKN_ENTRY:
             {
-                vfs_path_t *vpath;
                 CHECK_TOKEN (TKN_STRING);
                 label = g_strdup (tkn_buf->str);
                 CHECK_TOKEN (TKN_URL);
                 CHECK_TOKEN (TKN_STRING);
-                vpath = vfs_path_from_str_flags (tkn_buf->str, VPF_USE_DEPRECATED_PARSER);
-                url = vfs_path_to_str (vpath);
-                vfs_path_free (vpath);
+                url = g_strdup (tkn_buf->str);
                 add2hotlist (label, url, HL_TYPE_ENTRY, LISTBOX_APPEND_AT_END);
                 SKIP_TO_EOL;
             }
@@ -1696,14 +1689,18 @@ add_dotdot_to_list (void)
 void
 add2hotlist_cmd (void)
 {
-    char *lc_prompt, *label;
-    const char *cp = _("Label for \"%s\":");
-    int l = str_term_width1 (cp);
-    char *label_string = g_strdup (current_panel->cwd);
+    char *lc_prompt;
+    const char *cp = N_("Label for \"%s\":");
+    int l;
+    char *label_string, *label;
 
-    strip_password (label_string, 1);
+#ifdef ENABLE_NLS
+    cp = _(cp);
+#endif
 
-    lc_prompt = g_strdup_printf (cp, path_trunc (current_panel->cwd, COLS - 2 * UX - (l + 8)));
+    l = str_term_width1 (cp);
+    label_string = vfs_path_to_str_flags (current_panel->cwd_vpath, 0, VPF_STRIP_PASSWORD);
+    lc_prompt = g_strdup_printf (cp, str_trunc (label_string, COLS - 2 * UX - (l + 8)));
     label = input_dialog (_("Add to hotlist"), lc_prompt, MC_HISTORY_HOTLIST_ADD, label_string);
     g_free (lc_prompt);
 
