@@ -515,7 +515,7 @@ init_layout (void)
 
     layout_dlg =
         create_dlg (TRUE, 0, 0, 14, width,
-                    dialog_colors, layout_callback, "[Layout]",
+                    dialog_colors, layout_callback, NULL, "[Layout]",
                     _("Layout"), DLG_CENTER | DLG_REVERSE);
 
     /* buttons */
@@ -557,7 +557,7 @@ init_layout (void)
     }
 
     equal_split = panels_layout.horizontal_split ?
-                        panels_layout.horizontal_equal : panels_layout.vertical_equal;
+        panels_layout.horizontal_equal : panels_layout.vertical_equal;
 
     /* "Panel split" groupbox */
     bright_widget = button_new (6, 14, B_2RIGHT, NARROW_BUTTON, "&>", b_left_right_cback);
