@@ -212,6 +212,9 @@ create_panel_menu (void)
 #ifdef ENABLE_VFS_FISH
     entries = g_list_prepend (entries, menu_entry_create (_("S&hell link..."), CK_ConnectFish));
 #endif
+#ifdef ENABLE_VFS_SFTP
+    entries = g_list_prepend (entries, menu_entry_create (_("S&FTP link..."), CK_ConnectSftp));
+#endif
 #ifdef ENABLE_VFS_SMB
     entries = g_list_prepend (entries, menu_entry_create (_("SM&B link..."), CK_ConnectSmb));
 #endif
@@ -567,60 +570,81 @@ print_vfs_message (const gchar * event_group_name, const gchar * event_name,
 static void
 create_panels (void)
 {
-    int current_index;
-    int other_index;
+    int current_index, other_index;
     panel_view_mode_t current_mode, other_mode;
-    vfs_path_t *original_dir = NULL;
+    char *current_dir, *other_dir;
+    vfs_path_t *original_dir;
 
     if (boot_current_is_left)
     {
+        /* left panel is active */
         current_index = 0;
         other_index = 1;
         current_mode = startup_left_mode;
         other_mode = startup_right_mode;
+        /* if mc_run_param0 is NULL, working directory will be used for the left panel */
+        current_dir = (char *) mc_run_param0;
+        /* mc_run_param1 is never NULL. It is setup from command line or from panels.ini
+         * (value of other_dir). mc_run_param1 will be used for the right panel */
+        other_dir = mc_run_param1;
     }
     else
     {
+        /* right panel is active */
         current_index = 1;
         other_index = 0;
         current_mode = startup_right_mode;
         other_mode = startup_left_mode;
+
+        /* if mc_run_param0 is not NULL (it was setup from command line), it will be used
+         * for the left panel, working directory will be used for the right one;
+         * if mc_run_param0 is NULL, working directory will be used for the right (active) panel,
+         * mc_run_param1 will be used for the left one */
+        if (mc_run_param0 != NULL)
+        {
+            current_dir = NULL;
+            other_dir = (char *) mc_run_param0;
+        }
+        else
+        {
+            current_dir = NULL;
+            other_dir = mc_run_param1;
+        }
     }
-    /* Creates the left panel */
-    if (mc_run_param0 != NULL)
+
+    /* 1. Get current dir */
+    original_dir = vfs_path_clone (vfs_get_raw_current_dir ());
+
+    /* 2. Create passive panel */
+    if (other_dir != NULL)
     {
         vfs_path_t *vpath;
 
-        if (mc_run_param1 != NULL)
-        {
-            /* Ok, user has specified two dirs, save the original one,
-             * since we may not be able to chdir to the proper
-             * second directory later
-             */
-            original_dir = vfs_path_clone (vfs_get_raw_current_dir ());
-        }
-        vpath = vfs_path_from_str (mc_run_param0);
+        if (g_path_is_absolute (other_dir))
+            vpath = vfs_path_from_str (other_dir);
+        else
+            vpath = vfs_path_append_new (original_dir, other_dir, (char *) NULL);
+        mc_chdir (vpath);
+        vfs_path_free (vpath);
+    }
+    set_display_type (other_index, other_mode);
+
+    /* 3. Create active panel */
+    if (current_dir == NULL)
+        mc_chdir (original_dir);
+    else
+    {
+        vfs_path_t *vpath;
+
+        if (g_path_is_absolute (current_dir))
+            vpath = vfs_path_from_str (current_dir);
+        else
+            vpath = vfs_path_append_new (original_dir, current_dir, (char *) NULL);
         mc_chdir (vpath);
         vfs_path_free (vpath);
     }
     set_display_type (current_index, current_mode);
 
-    /* The other panel */
-    if (mc_run_param1 != NULL)
-    {
-        vfs_path_t *vpath;
-
-        if (original_dir != NULL)
-            mc_chdir (original_dir);
-
-        vpath = vfs_path_from_str (mc_run_param1);
-        mc_chdir (vpath);
-        vfs_path_free (vpath);
-    }
-    vfs_path_free (original_dir);
-
-    set_display_type (other_index, other_mode);
-
     if (startup_left_mode == view_listing)
         current_panel = left_panel;
     else if (right_panel != NULL)
@@ -628,6 +652,8 @@ create_panels (void)
     else
         current_panel = left_panel;
 
+    vfs_path_free (original_dir);
+
 #ifdef ENABLE_VFS
     mc_event_add (MCEVENT_GROUP_CORE, "vfs_timestamp", check_other_panel_timestamp, NULL, NULL);
     mc_event_add (MCEVENT_GROUP_CORE, "vfs_timestamp", check_current_panel_timestamp, NULL, NULL);
@@ -655,20 +681,27 @@ static void
 put_current_path (void)
 {
     char *cwd_path;
-    vfs_path_t *cwd_vpath;
 
     if (!command_prompt)
         return;
 
-    cwd_vpath = remove_encoding_from_path (current_panel->cwd_vpath);
-    cwd_path = vfs_path_to_str (cwd_vpath);
-    command_insert (cmdline, cwd_path, FALSE);
+#ifdef HAVE_CHARSET
+    {
+        vfs_path_t *cwd_vpath;
 
+        cwd_vpath = remove_encoding_from_path (current_panel->cwd_vpath);
+        cwd_path = vfs_path_to_str (cwd_vpath);
+        vfs_path_free (cwd_vpath);
+    }
+#else
+    cwd_path = vfs_path_to_str (current_panel->cwd_vpath);
+#endif
+
+    command_insert (cmdline, cwd_path, FALSE);
     if (cwd_path[strlen (cwd_path) - 1] != PATH_SEP)
         command_insert (cmdline, PATH_SEP_STR, FALSE);
 
     g_free (cwd_path);
-    vfs_path_free (cwd_vpath);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -677,7 +710,6 @@ static void
 put_other_path (void)
 {
     char *cwd_path;
-    vfs_path_t *cwd_vpath;
 
     if (get_other_type () != view_listing)
         return;
@@ -685,15 +717,23 @@ put_other_path (void)
     if (!command_prompt)
         return;
 
-    cwd_vpath = remove_encoding_from_path (other_panel->cwd_vpath);
-    cwd_path = vfs_path_to_str (cwd_vpath);
-    command_insert (cmdline, cwd_path, FALSE);
+#ifdef HAVE_CHARSET
+    {
+        vfs_path_t *cwd_vpath;
 
+        cwd_vpath = remove_encoding_from_path (other_panel->cwd_vpath);
+        cwd_path = vfs_path_to_str (cwd_vpath);
+        vfs_path_free (cwd_vpath);
+    }
+#else
+    cwd_path = vfs_path_to_str (other_panel->cwd_vpath);
+#endif
+
+    command_insert (cmdline, cwd_path, FALSE);
     if (cwd_path[strlen (cwd_path) - 1] != PATH_SEP)
         command_insert (cmdline, PATH_SEP_STR, FALSE);
 
     g_free (cwd_path);
-    vfs_path_free (cwd_vpath);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -947,33 +987,24 @@ prepend_cwd_on_local (const char *filename)
 static gboolean
 mc_maybe_editor_or_viewer (void)
 {
-    int ret;
+    gboolean ret;
 
     switch (mc_global.mc_run_mode)
     {
 #ifdef USE_INTERNAL_EDIT
     case MC_RUN_EDITOR:
-        {
-            vfs_path_t *vpath = NULL;
-
-            if (mc_run_param0 != NULL && *mc_run_param0 != '\0')
-                vpath = vfs_path_from_str (mc_run_param0);
-
-            ret = edit_file (vpath, mc_args__edit_start_line);
-            vfs_path_free (vpath);
-        }
+        ret = edit_files ((GList *) mc_run_param0);
         break;
 #endif /* USE_INTERNAL_EDIT */
     case MC_RUN_VIEWER:
         {
             vfs_path_t *vpath = NULL;
 
-            if (mc_run_param0 != NULL && *mc_run_param0 != '\0')
-                vpath = prepend_cwd_on_local (mc_run_param0);
+            if (mc_run_param0 != NULL && *(char *) mc_run_param0 != '\0')
+                vpath = prepend_cwd_on_local ((char *) mc_run_param0);
 
-            view_file (vpath, 0, 1);
+            ret = view_file (vpath, 0, 1);
             vfs_path_free (vpath);
-            ret = 1;
             break;
         }
 #ifdef USE_DIFF_VIEW
@@ -982,10 +1013,10 @@ mc_maybe_editor_or_viewer (void)
         break;
 #endif /* USE_DIFF_VIEW */
     default:
-        ret = 0;
+        ret = FALSE;
     }
 
-    return (ret != 0);
+    return ret;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1187,6 +1218,11 @@ midnight_execute_cmd (Widget * sender, unsigned long command)
         ftplink_cmd ();
         break;
 #endif
+#ifdef ENABLE_VFS_SFTP
+    case CK_ConnectSftp:
+        sftplink_cmd ();
+        break;
+#endif
 #ifdef ENABLE_VFS_SMB
     case CK_ConnectSmb:
         smblink_cmd ();
@@ -1561,6 +1597,50 @@ midnight_callback (Dlg_head * h, Widget * sender, dlg_msg_t msg, int parm, void
     }
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
+static int
+midnight_event (Gpm_Event * event, void *data)
+{
+    Dlg_head *h = (Dlg_head *) data;
+    int ret = MOU_UNHANDLED;
+
+    if (event->y == h->y + 1)
+    {
+        /* menubar */
+        if (menubar_visible || the_menubar->is_active)
+            ret = ((Widget *) the_menubar)->mouse (event, the_menubar);
+        else
+        {
+            Widget *w;
+
+            w = get_panel_widget (0);
+            if (w->mouse != NULL)
+                ret = w->mouse (event, w);
+
+            if (ret == MOU_UNHANDLED)
+            {
+                w = get_panel_widget (1);
+                if (w->mouse != NULL)
+                    ret = w->mouse (event, w);
+            }
+
+            if (ret == MOU_UNHANDLED)
+                ret = ((Widget *) the_menubar)->mouse (event, the_menubar);
+        }
+    }
+    else if (event->y == h->y + h->lines && mc_global.keybar_visible)
+    {
+        /* buttonbar */
+
+        /* in general, this can be handled in default way (dlg_mouse_event)
+         * but let make it here to avoid walking in widget list */
+        ret = ((Widget *) the_bar)->mouse (event, the_bar);
+    }
+
+    return ret;
+}
+
 /* --------------------------------------------------------------------------------------------- */
 /*** public functions ****************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
@@ -1574,6 +1654,8 @@ update_menu (void)
     menubar_set_visible (the_menubar, menubar_visible);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 void
 midnight_set_buttonbar (WButtonBar * b)
 {
@@ -1681,7 +1763,7 @@ do_nc (void)
 #endif
 
     midnight_dlg = create_dlg (FALSE, 0, 0, LINES, COLS, midnight_colors, midnight_callback,
-                               "[main]", NULL, DLG_WANT_IDLE);
+                               midnight_event, "[main]", NULL, DLG_WANT_IDLE);
 
     if (mc_global.mc_run_mode == MC_RUN_FULL)
         setup_mc ();
