@@ -1,20 +1,28 @@
-/* User interface for charset selection.
+/*
+   User interface for charset selection.
 
    Copyright (C) 2001 Walery Studennikov <despair@sama.ru>
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   Copyright (C) 2011
+   The Free Software Foundation, Inc.
 
-   This program is distributed in the hope that it will be useful,
+   Written by:
+   Walery Studennikov <despair@sama.ru>, 2001
+
+   This file is part of the Midnight Commander.
+
+   The Midnight Commander is free software: you can redistribute it
+   and/or modify it under the terms of the GNU General Public License as
+   published by the Free Software Foundation, either version 3 of the License,
+   or (at your option) any later version.
+
+   The Midnight Commander is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /** \file selcodepage.c
@@ -23,8 +31,6 @@
 
 #include <config.h>
 
-#ifdef HAVE_CHARSET
-
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -136,9 +142,10 @@ do_set_codepage (int codepage)
     char *errmsg;
     gboolean ret;
 
-    source_codepage = codepage;
+    mc_global.source_codepage = codepage;
     errmsg = init_translation_table (codepage == SELECT_CHARSET_NO_TRANSLATE ?
-                                     display_codepage : source_codepage, display_codepage);
+                                     mc_global.display_codepage : mc_global.source_codepage,
+                                     mc_global.display_codepage);
     ret = errmsg == NULL;
 
     if (!ret)
@@ -167,5 +174,3 @@ do_select_codepage (void)
 }
 
 /* --------------------------------------------------------------------------------------------- */
-
-#endif /* HAVE_CHARSET */
