@@ -774,7 +774,7 @@ init_subshell (void)
         return;
     case 2:
         mc_global.tty.use_subshell = FALSE;
-        mc_global.widget.midnight_shutdown = TRUE;
+        mc_global.midnight_shutdown = TRUE;
         return;
     }
 
@@ -918,7 +918,7 @@ init_subshell (void)
 /* --------------------------------------------------------------------------------------------- */
 
 int
-invoke_subshell (const char *command, int how, char **new_dir)
+invoke_subshell (const char *command, int how, vfs_path_t ** new_dir_vpath)
 {
     char *pcwd;
 
@@ -926,8 +926,8 @@ invoke_subshell (const char *command, int how, char **new_dir)
     tcsetattr (STDOUT_FILENO, TCSANOW, &raw_mode);
 
     /* Make the subshell change to MC's working directory */
-    if (new_dir != NULL)
-        do_subshell_chdir (current_panel->cwd, TRUE, TRUE);
+    if (new_dir_vpath != NULL)
+        do_subshell_chdir (current_panel->cwd_vpath, TRUE, TRUE);
 
     if (command == NULL)        /* The user has done "C-o" from MC */
     {
@@ -953,9 +953,16 @@ invoke_subshell (const char *command, int how, char **new_dir)
 
     feed_subshell (how, FALSE);
 
-    pcwd = vfs_translate_path_n (current_panel->cwd);
-    if (new_dir && subshell_alive && strcmp (subshell_cwd, pcwd))
-        *new_dir = subshell_cwd;        /* Make MC change to the subshell's CWD */
+    {
+        char *cwd_str;
+
+        cwd_str = vfs_path_to_str (current_panel->cwd_vpath);
+        pcwd = vfs_translate_path_n (cwd_str);
+        g_free (cwd_str);
+    }
+
+    if (new_dir_vpath != NULL && subshell_alive && strcmp (subshell_cwd, pcwd))
+        *new_dir_vpath = vfs_path_from_str (subshell_cwd);      /* Make MC change to the subshell's CWD */
     g_free (pcwd);
 
     /* Restart the subshell if it has died by SIGHUP, SIGQUIT, etc. */
@@ -1161,13 +1168,13 @@ subshell_name_quote (const char *s)
 
 /** If it actually changed the directory it returns true */
 void
-do_subshell_chdir (const char *directory, gboolean update_prompt, gboolean reset_prompt)
+do_subshell_chdir (const vfs_path_t * vpath, gboolean update_prompt, gboolean reset_prompt)
 {
     char *pcwd;
     char *temp;
-    char *translate;
+    char *directory;
 
-    pcwd = vfs_translate_path_n (current_panel->cwd);
+    pcwd = vfs_path_to_str_flags (current_panel->cwd_vpath, 0, VPF_RECODE);
 
     if (!(subshell_state == INACTIVE && strcmp (subshell_cwd, pcwd) != 0))
     {
@@ -1184,10 +1191,14 @@ do_subshell_chdir (const char *directory, gboolean update_prompt, gboolean reset
     /* The initial space keeps this out of the command history (in bash
        because we set "HISTCONTROL=ignorespace") */
     write_all (mc_global.tty.subshell_pty, " cd ", 4);
-    if (*directory)
+
+    directory = vfs_path_to_str (vpath);
+    if (directory != '\0')
     {
+        char *translate;
+
         translate = vfs_translate_path_n (directory);
-        if (translate)
+        if (translate != NULL)
         {
             temp = subshell_name_quote (translate);
             if (temp)
@@ -1212,6 +1223,7 @@ do_subshell_chdir (const char *directory, gboolean update_prompt, gboolean reset
     {
         write_all (mc_global.tty.subshell_pty, "/", 1);
     }
+    g_free (directory);
     write_all (mc_global.tty.subshell_pty, "\n", 1);
 
     subshell_state = RUNNING_COMMAND;
@@ -1236,10 +1248,12 @@ do_subshell_chdir (const char *directory, gboolean update_prompt, gboolean reset
             bPathNotEq = strcmp (p_subshell_cwd, p_current_panel_cwd);
         }
 
-        if (bPathNotEq && strcmp (pcwd, "."))
+        if (bPathNotEq && strcmp (pcwd, ".") != 0)
         {
-            char *cwd = strip_password (g_strdup (pcwd), 1);
-            fprintf (stderr, _("Warning: Cannot change to %s.\n"), cwd);
+            char *cwd;
+
+            cwd = vfs_path_to_str_flags (current_panel->cwd_vpath, 0, VPF_STRIP_PASSWORD);
+            vfs_print_message (_("Warning: Cannot change to %s.\n"), cwd);
             g_free (cwd);
         }
     }
@@ -1264,7 +1278,6 @@ subshell_get_console_attributes (void)
     {
         fprintf (stderr, "Cannot get terminal settings: %s\r\n", unix_error_string (errno));
         mc_global.tty.use_subshell = FALSE;
-        return;
     }
 }
 
