@@ -48,16 +48,26 @@
 /*** public functions ****************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
 
-int
-check_for_default (const char *default_file, const char *file)
+gboolean
+check_for_default (const vfs_path_t * default_file_vpath, const vfs_path_t * file_vpath)
 {
+    char *file;
+
+    file = vfs_path_to_str (file_vpath);
+
     if (!exist_file (file))
     {
+        char *default_file;
         FileOpContext *ctx;
         FileOpTotalContext *tctx;
 
+        default_file = vfs_path_to_str (default_file_vpath);
         if (!exist_file (default_file))
-            return -1;
+        {
+            g_free (file);
+            g_free (default_file);
+            return FALSE;
+        }
 
         ctx = file_op_context_new (OP_COPY);
         tctx = file_op_total_context_new ();
@@ -65,9 +75,12 @@ check_for_default (const char *default_file, const char *file)
         copy_file_file (tctx, ctx, default_file, file);
         file_op_total_context_destroy (tctx);
         file_op_context_destroy (ctx);
+        g_free (default_file);
     }
 
-    return 0;
+    g_free (file);
+
+    return TRUE;
 }
 
 /* --------------------------------------------------------------------------------------------- */
