@@ -249,7 +249,7 @@ cpio_open_cpio_file (struct vfs_class *me, struct vfs_s_super *super, const vfs_
 
         mc_close (fd);
         s = g_strconcat (super->name, decompress_extension (type), (char *) NULL);
-        tmp_vpath = vfs_path_from_str (s);
+        tmp_vpath = vfs_path_from_str_flags (s, VPF_NO_CANON);
         fd = mc_open (tmp_vpath, O_RDONLY);
         vfs_path_free (tmp_vpath);
         if (fd == -1)
@@ -854,7 +854,7 @@ init_cpiofs (void)
 {
     static struct vfs_s_subclass cpio_subclass;
 
-    cpio_subclass.flags = VFS_S_READONLY;        /* FIXME: cpiofs used own temp files */
+    cpio_subclass.flags = VFS_S_READONLY;       /* FIXME: cpiofs used own temp files */
     cpio_subclass.archive_check = cpio_super_check;
     cpio_subclass.archive_same = cpio_super_same;
     cpio_subclass.open_archive = cpio_open_archive;
