@@ -12,6 +12,7 @@
 #include "lib/vfs/vfs.h"        /* vfs_path_t */
 
 #include "src/keybind-defaults.h"       /* global_keymap_t */
+#include "src/filemanager/dir.h"        /* dir_list */
 
 /*** typedefs(not structures) and defined constants **********************************************/
 
@@ -163,12 +164,6 @@ typedef struct mcview_struct
     int marker;                 /* mark to use */
     off_t marks[10];            /* 10 marks: 0..9 */
 
-    int move_dir;               /* return value from widget:
-                                 * 0 do nothing
-                                 * -1 view previous file
-                                 * 1 view next file
-                                 */
-
     off_t update_steps;         /* The number of bytes between percent
                                  * increments */
     off_t update_activate;      /* Last point where we updated the status */
@@ -184,6 +179,13 @@ typedef struct mcview_struct
     int search_numNeedSkipChar;
 
     GArray *saved_bookmarks;
+
+    dir_list *dir;              /* List of current directory files
+                                 * to handle CK_FileNext and CK_FilePrev commands */
+    int *dir_count;             /* Number of files in dir structure.
+                                 * Pointer is used here as reference to WPanel::count */
+    int *dir_idx;               /* Index of current file in dir structure.
+                                 * Pointer is used here as reference to WPanel::count */
 } mcview_t;
 
 typedef struct mcview_nroff_struct
@@ -321,7 +323,8 @@ int mcview_nroff_seq_prev (mcview_nroff_t *);
 void mcview_display_text (mcview_t *);
 
 /* search.c: */
-int mcview_search_cmd_callback (const void *user_data, gsize char_offset);
+mc_search_cbret_t mcview_search_cmd_callback (const void *user_data, gsize char_offset,
+                                              int *current_char);
 int mcview_search_update_cmd_callback (const void *, gsize);
 void mcview_do_search (mcview_t * view);
 
