@@ -45,7 +45,6 @@
 #include "lib/strutil.h"
 #include "lib/util.h"           /* load_file_position() */
 #include "lib/widget.h"
-#include "lib/charsets.h"
 
 #include "src/main.h"
 
@@ -93,18 +92,18 @@ do_mcview_event (mcview_t * view, Gpm_Event * event, int *result)
 {
     screen_dimen y, x;
     Gpm_Event local;
+    Widget *w = (Widget *) view;
+
+    /* rest of the upper frame - call menu */
+    if (mcview_is_in_panel (view) && (event->type & GPM_DOWN) != 0 && event->y == w->owner->y + 1)
+    {
+        *result = MOU_UNHANDLED;
+        return FALSE;           /* don't draw viewer over menu */
+    }
 
     *result = MOU_NORMAL;
 
-    local = mouse_get_local (event, (Widget *) view);
-
-    /* rest of the upper frame, the menu is invisible - call menu */
-    if (mcview_is_in_panel (view) && (local.type & GPM_DOWN) != 0 && local.y == 1
-        && !menubar_visible)
-    {
-        *result = the_menubar->widget.mouse (event, the_menubar);
-        return FALSE;           /* don't draw viewer over menu */
-    }
+    local = mouse_get_local (event, w);
 
     /* We are not interested in the release events */
     if ((local.type & (GPM_DOWN | GPM_DRAG)) == 0)
@@ -114,13 +113,11 @@ do_mcview_event (mcview_t * view, Gpm_Event * event, int *result)
     if ((local.buttons & GPM_B_UP) != 0 && (local.type & GPM_DOWN) != 0)
     {
         mcview_move_up (view, 2);
-        *result = MOU_NORMAL;
         return TRUE;
     }
     if ((local.buttons & GPM_B_DOWN) != 0 && (local.type & GPM_DOWN) != 0)
     {
         mcview_move_down (view, 2);
-        *result = MOU_NORMAL;
         return TRUE;
     }
 
@@ -135,7 +132,8 @@ do_mcview_event (mcview_t * view, Gpm_Event * event, int *result)
             mcview_move_left (view, 1);
             goto processed;
         }
-        else if (x < view->data_area.width * 3 / 4)
+
+        if (x < view->data_area.width * 3 / 4)
         {
             /* ignore the click */
         }
@@ -231,16 +229,15 @@ mcview_new (int y, int x, int lines, int cols, gboolean is_panel)
 /* --------------------------------------------------------------------------------------------- */
 /** Real view only */
 
-mcview_ret_t
+gboolean
 mcview_viewer (const char *command, const vfs_path_t * file_vpath, int start_line)
 {
     gboolean succeeded;
     mcview_t *lc_mcview;
     Dlg_head *view_dlg;
-    mcview_ret_t ret;
 
     /* Create dialog and widgets, put them on the dialog */
-    view_dlg = create_dlg (FALSE, 0, 0, LINES, COLS, NULL, mcview_dialog_callback,
+    view_dlg = create_dlg (FALSE, 0, 0, LINES, COLS, NULL, mcview_dialog_callback, NULL,
                            "[Internal File Viewer]", NULL, DLG_WANT_TAB);
 
     lc_mcview = mcview_new (0, 0, LINES - 1, COLS, FALSE);
@@ -259,22 +256,14 @@ mcview_viewer (const char *command, const vfs_path_t * file_vpath, int start_lin
     }
 
     if (succeeded)
-    {
         run_dlg (view_dlg);
-
-        ret = lc_mcview->move_dir == 0 ? MCVIEW_EXIT_OK :
-            lc_mcview->move_dir > 0 ? MCVIEW_WANT_NEXT : MCVIEW_WANT_PREV;
-    }
     else
-    {
         view_dlg->state = DLG_CLOSED;
-        ret = MCVIEW_EXIT_FAILURE;
-    }
 
     if (view_dlg->state == DLG_CLOSED)
         destroy_dlg (view_dlg);
 
-    return ret;
+    return succeeded;
 }
 
 /* {{{ Miscellaneous functions }}} */
@@ -293,23 +282,29 @@ mcview_load (mcview_t * view, const char *command, const char *file, int start_l
 
     view->filename_vpath = vfs_path_from_str (file);
 
-    if ((view->workdir_vpath == NULL) && (file != NULL))
+    /* get working dir */
+    if (file != NULL && file[0] != '\0')
     {
+        vfs_path_free (view->workdir_vpath);
+
         if (!g_path_is_absolute (file))
-            view->workdir_vpath = vfs_path_clone (vfs_get_raw_current_dir ());
+        {
+            vfs_path_t *p;
+
+            p = vfs_path_clone (vfs_get_raw_current_dir ());
+            view->workdir_vpath = vfs_path_append_new (p, file, (char *) NULL);
+            vfs_path_free (p);
+        }
         else
         {
             /* try extract path form filename */
-            char *dirname;
+            const char *fname;
+            char *dir;
 
-            dirname = g_path_get_dirname (file);
-            if (strcmp (dirname, ".") != 0)
-                view->workdir_vpath = vfs_path_from_str (dirname);
-            else
-            {
-                view->workdir_vpath = vfs_path_clone (vfs_get_raw_current_dir ());
-            }
-            g_free (dirname);
+            fname = x_basename (file);
+            dir = g_strndup (file, (size_t) (fname - file));
+            view->workdir_vpath = vfs_path_from_str (dir);
+            g_free (dir);
         }
     }
 
