@@ -26,11 +26,9 @@
 
 #include "lib/global.c"
 
-#ifndef HAVE_CHARSET
-#define HAVE_CHARSET 1
-#endif
-
+#ifdef HAVE_CHARSET
 #include "lib/charsets.h"
+#endif
 
 #include "lib/strutil.h"
 #include "lib/vfs/xdirentry.h"
@@ -49,27 +47,31 @@ setup (void)
     vfs_setup_work_dir ();
 
     mc_global.sysconfig_dir = (char *) TEST_SHARE_DIR;
+#ifdef HAVE_CHARSET
     load_codepages_list ();
+#endif
 }
 
 static void
 teardown (void)
 {
+#ifdef HAVE_CHARSET
     free_codepages_list ();
+#endif
 
     vfs_shut ();
     str_uninit_strings ();
 }
 
 /* --------------------------------------------------------------------------------------------- */
-#define path_cmp_one_check(input1, input2, etalon) {\
+#define path_cmp_one_check(input1, input2, etalon_condition) {\
     vpath1 = vfs_path_from_str (input1);\
     vpath2 = vfs_path_from_str (input2);\
     result = vfs_path_cmp (vpath1, vpath2);\
     vfs_path_free (vpath1); \
     vfs_path_free (vpath2); \
-    fail_unless ( result == etalon, "\ninput1: %s\ninput2: %s\nexpected: %d\nactual: %d\n",\
-        input1, input2, etalon, result); \
+    fail_unless ( result etalon_condition, "\ninput1: %s\ninput2: %s\nexpected: %d\nactual: %d\n",\
+        input1, input2, #etalon_condition, result); \
 }
 
 START_TEST (test_path_compare)
@@ -77,11 +79,12 @@ START_TEST (test_path_compare)
     vfs_path_t *vpath1, *vpath2;
     int result;
 
-    path_cmp_one_check ("/тестовый/путь", "/тестовый/путь", 0);
+    path_cmp_one_check ("/тестовый/путь", "/тестовый/путь", ==0);
 
-    path_cmp_one_check ("/#enc:KOI8-R/тестовый/путь", "/тестовый/путь", -174);
-
-    path_cmp_one_check ("/тестовый/путь", "/#enc:KOI8-R/тестовый/путь", 174);
+#ifdef HAVE_CHARSET
+    path_cmp_one_check ("/#enc:KOI8-R/тестовый/путь", "/тестовый/путь", <0);
+    path_cmp_one_check ("/тестовый/путь", "/#enc:KOI8-R/тестовый/путь", >0);
+#endif
 
     path_cmp_one_check (NULL, "/тестовый/путь", -1);
     path_cmp_one_check ("/тестовый/путь", NULL, -1);
@@ -92,14 +95,14 @@ END_TEST
 /* --------------------------------------------------------------------------------------------- */
 #undef path_cmp_one_check
 
-#define path_cmp_one_check(input1, input2, len, etalon) {\
+#define path_cmp_one_check(input1, input2, len, etalon_condition) {\
     vpath1 = vfs_path_from_str (input1);\
     vpath2 = vfs_path_from_str (input2);\
     result = vfs_path_ncmp (vpath1, vpath2, len);\
     vfs_path_free (vpath1); \
     vfs_path_free (vpath2); \
-    fail_unless ( result == etalon, "\ninput1: %s\ninput2: %s\nexpected: %d\nactual: %d\n",\
-        input1, input2, etalon, result); \
+    fail_unless ( result etalon_condition, "\ninput1: %s\ninput2: %s\nexpected: %d\nactual: %d\n",\
+        input1, input2, #etalon_condition, result); \
 }
 
 START_TEST (test_path_compare_len)
@@ -107,17 +110,17 @@ START_TEST (test_path_compare_len)
     vfs_path_t *vpath1, *vpath2;
     int result;
 
-    path_cmp_one_check ("/тестовый/путь", "/тестовый/путь", 10, 0);
+    path_cmp_one_check ("/тестовый/путь", "/тестовый/путь", 10, ==0);
 
-    path_cmp_one_check ("/тест/овый/путь", "/тестовый/путь", 10, -161);
+    path_cmp_one_check ("/тест/овый/путь", "/тестовый/путь", 10, <0);
 
-    path_cmp_one_check ("/тестовый/путь", "/тест/овый/путь", 10, 161);
+    path_cmp_one_check ("/тестовый/путь", "/тест/овый/путь", 10, >0);
 
-    path_cmp_one_check ("/тест/овый/путь", "/тестовый/путь", 9, 0);
+    path_cmp_one_check ("/тест/овый/путь", "/тестовый/путь", 9, ==0);
 
-    path_cmp_one_check (NULL, "/тестовый/путь", 0, -1);
-    path_cmp_one_check ("/тестовый/путь", NULL, 0, -1);
-    path_cmp_one_check (NULL, NULL, 0, -1);
+    path_cmp_one_check (NULL, "/тестовый/путь", 0, <0);
+    path_cmp_one_check ("/тестовый/путь", NULL, 0, <0);
+    path_cmp_one_check (NULL, NULL, 0, <0);
 }
 END_TEST
 
