@@ -31,10 +31,6 @@
 
 #include "lib/global.c"
 
-#ifndef HAVE_CHARSET
-#define HAVE_CHARSET 1
-#endif
-
 #include "lib/charsets.h"
 
 #include "lib/strutil.h"
